/*********************************************************
 * MOIRCS acquisition:
 *
 *    Determin center using pixel value above fThresh
 *       < centroid, i.e. 1st order morment >
 *
 *       Input  : *pix_val  :
 *       Input  : iNx       : x-dimension of pix_val
 *       Input  : iNy       : y-dimention of pix_val
 *       Input  : fRcen     : 
 *       Input  : fCenOld_x :
 *       Input  : fCenOld_y :  
 *       Input  : fThresh   : Threshold for determining center
 *       Output : fCenNew_x :
 *       Output : fCenNew_y :
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

void ac_center2(const float *pix_val, const int *mask_val,
             const int iNx, const int iNy,
             const float fRcen,
             const float fCenOld_x, const float fCenOld_y,
             const float fThresh,
             float *fCenNew_x, float *fCenNew_y)
{  

   int     i,j,k;
   float   fCen1_x, fCen1_y;
   float   fDiff_x, fDiff_y, fDiff;
   float   fCen2_x, fCen2_y;
   float   fSum_x,fSum_y,fMom_x,fMom_y;
   float   fDist;
   float   fScale;
   int     iNite;

   int     iNpix;

#ifdef DEBUG_AC_CENTER2
   printf("ac_center2 : Centering with ac_center1 in r=%6.2f pix region \n", fRcen);
#endif

   fCen1_x = fCenOld_x;
   fCen1_y = fCenOld_y;
   fDiff = 2.0;
 
   iNite = 0;

    fMom_x = 0.0;
    fMom_y = 0.0;
    fSum_x = 0.0;
    fSum_y = 0.0;

    fScale = 1.0;

/*
 * Determine Center pixel value above fThresh within fRcen
 *   0th order moment
 */ 
    iNpix = MINNPIX + 10;
    while( fDiff >= 0.5 && iNpix>MINNPIX ){
        iNpix = 0;
        for (j=1; j<=iNy; j++){
            for (i=1; i<=iNx; i++){
                k = (j-1)*iNx + (i-1); /* Fortran convention */
   
                if (*(pix_val+k) > fThresh && *(mask_val+k)!=MASK_VALUE){

                   fDist = sqrt((i*fScale - fCen1_x + 0.5)*(i*fScale - fCen1_x + 0.5)
                                 +(j*fScale - fCen1_y + 0.5)*(j*fScale - fCen1_y + 0.5));
                   iNpix = iNpix + 1;
 
                   if ( fDist <= fRcen ){
                       fMom_x = fMom_x + i* *(pix_val+k);
                       fMom_y = fMom_y + j* *(pix_val+k);
                       fSum_x = fSum_x + *(pix_val+k);
                       fSum_y = fSum_y + *(pix_val+k);
                   }
                }
            }
        }
        fCen2_x = fCen1_x;
        fCen2_y = fCen1_y;

        fCen1_x = fMom_x / fSum_x;
        fCen1_y = fMom_y / fSum_y;

        iNite = iNite + 1;

#ifdef DEBUG_AC_CENTER2
        printf(" ac_center2 : iNite = %d x = %f y = %f \n", iNite, fCen1_x, fCen1_y);
#endif

        fDiff_x = fCen1_x - fCen2_x;
        fDiff_y = fCen1_y - fCen2_y;

        fDiff = sqrt( fDiff_x * fDiff_x + fDiff_y * fDiff_y );
    }

    if( iNpix>MINNPIX ){
       *fCenNew_x = fCen1_x;
       *fCenNew_y = fCen1_y;
    }
    else{
#ifdef DEBUG_AC_CENTER2
    fprintf(stderr, "ac_center2 : Centering failed............................\n" );
#endif
       *fCenNew_x = 0.0;
       *fCenNew_y = 0.0;
    }

    return;
}
