/*********************************************************
 * Aperture Photometory:
 *
 *    Correct Bad Pixel Region
 *
 *       Input  : *pPix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

int ac_corre_img2(float *pPix_val, int *pMask_val,
   const int iNx, const int iNy,
   const float fCen_X, const float fCen_Y, const float fRadius,
   const char *cLine, const int iInterac)
{
  int       i, j, k;
  float     fmin, fmax;
  static float tr[6] = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
  float     fX1, fX2, fY1, fY2;
  float     fXcur, fYcur;
  int       iMask_xmin, iMask_xmax, iMask_ymin, iMask_ymax;
  char      cEnd;
  char      cText[100];

  float     *pPix_val_in;

  int       iNin;
  float     fScale;
  float     fDist;

  float     sigma_rej;
  int       ndata;
  float     average,sigma;

   ndata = iNx * iNy;
   sigma_rej = REJECTION_SIGMA;

#ifdef DEBUG_AC_CORRE_IMG2
  fprintf(stderr, " ac_corre_img2 : Determine sigma and average in the hole region\n");
#endif
   pPix_val_in = (float *)malloc(sizeof(float)*(iNx * iNy));
   fScale = 1.0;
   iNin = 0;
   for(i=1; i<=iNx; i++){
      for(j=1; j<=iNy; j++){
         fDist = sqrt((i*fScale - fCen_X + 0.5)*(i*fScale - fCen_X + 0.5)
                      +(j*fScale - fCen_Y + 0.5)*(j*fScale - fCen_Y + 0.5));
         if( fDist < fRadius ){
             k = (j-1)*iNx + (i-1); /* Fortran convention */
             *(pPix_val_in+iNin) = *(pPix_val+k);
             iNin = iNin+1;
         }
      }
   }
   ac_sigclip(pPix_val_in, iNin, sigma_rej, 0, &average, &sigma);
 
#ifdef DEBUG_AC_CORRE_IMG2
   fprintf(stdout, "ac_corre_img2 : average = %f sigma = %f in the region with %5.2f sigma rejection \n",
                    average, sigma, sigma_rej);
#endif

#ifdef DEBUG_AC_CORRE_IMG2
  fprintf(stderr, " ac_corre_img2 : Subtract background constant value (average) \n");
  fprintf(stderr, " ac_corre_img2 : Replace edge region with 0 \n");
#endif
   for(i=1; i<=iNx; i++){
      for(j=1; j<=iNy; j++){
         k = (j-1)*iNx + (i-1); /* Fortran convention */
         *(pPix_val+k) = *(pPix_val+k)-average;

         fDist = sqrt((i*fScale - fCen_X + 0.5)*(i*fScale - fCen_X + 0.5)
                      +(j*fScale - fCen_Y + 0.5)*(j*fScale - fCen_Y + 0.5));
         if( fDist > MASK_EDGE*fRadius ){
             k = (j-1)*iNx + (i-1); /* Fortran convention */
             *(pPix_val+k) = 0.0;
             *(pMask_val+k) = MASKED_VALUE;
         }
      }
   }

/*
 * Display range
 */
   fmin = - VIEW_LOWSIGMA * sigma;
   fmax = VIEW_UPSIGMA * sigma;

   cEnd = 'n';
   if( iInterac == 0 ){
      cEnd = 88;
   }
     
while(cEnd != 88){
  cEnd = 'n';

/* 
 * Clear the screen. Set up window and viewport. 
 */
  cpgpage();
  cpgsvp(0.1, 0.7, 0.1, 0.1+0.6*1.2);
  cpgswin(1,iNx,1,iNy);
  cpggray(pPix_val, iNx, iNy, 1, iNx, 1, iNy, fmin , fmax, tr);

  cpgsch(1.0);
  cpgsci(1);
  sprintf(cText, "AC_CORRE_IMG : %s", cLine);
  cpgptxt( iNx*1.1, iNy*1.0, 0.0, 0.0, cText );
  sprintf(cText, "Remove Bad Pixel Region If needed");
  cpgptxt( iNx*1.1, iNy*0.9, 0.0, 0.0, cText );
  sprintf(cText, "Determin Mask Region ");
  cpgptxt( iNx*1.1, iNy*0.8, 0.0, 0.0, cText );
  sprintf(cText, "Click Left Twice, At bottom left and up right positions");
  cpgptxt( iNx*1.1, iNy*0.75, 0.0, 0.0, cText );
  sprintf(cText, "Click Right to Quit");
  cpgptxt( iNx*1.1, iNy*0.6, 0.0, 0.0, cText );

  cpgcurs(&fXcur, &fYcur, &cEnd);
  fX1 = fXcur;
  fY1 = fYcur; 

  if ( cEnd == 65 ){
      cpgcurs(&fXcur, &fYcur, &cEnd);
      fX2 = fXcur;
      fY2 = fYcur;

      if( fX1 > fX2 ){
        iMask_xmin = (int)fX2;
        iMask_xmax = (int)fX1;
      }
      else{
        iMask_xmin = (int)fX1;
        iMask_xmax = (int)fX2;
      }
      if( fY1 > fY2 ){
        iMask_ymin = (int)fY2;
        iMask_ymax = (int)fY1;
      }
      else{
        iMask_ymin = (int)fY1;
        iMask_ymax = (int)fY2;
      }

      if( iMask_xmin < 0 )  { iMask_xmin = 0; }
      if( iMask_xmax > iNx ){ iMask_xmax = iNx; }
      if( iMask_ymin < 0 )  { iMask_ymin = 0; }
      if( iMask_ymin > iNy ){ iMask_ymax = iNy; }
      
      for ( j=iMask_ymin; j <=iMask_ymax; j++ ){
          for ( i=iMask_xmin; i<=iMask_xmax; i++){
               k = (j-1)*iNx + (i-1);  /* Fortran convention */
               *(pMask_val+k) = MASK_VALUE ;
               *(pPix_val+k) = MASKED_VALUE ;
          }
      }
  }
  
  }
  free(pPix_val_in);
  return EXIT_SUCCESS;
}
