/*********************************************************
 * MOIRCS acquisition:
 *
 *    Determin radius of a hole using pixel value above fThresh
 *       < equal weight, not centroid >
 *
 *       Input  : *pix_val  :
 *       Input  : iNx       : x-dimension of pix_val
 *       Input  : iNy       : y-dimention of pix_val
 *       Input  : fRstat    : Radius of region used for statistics 
 *       Input  : fCenOld_x :
 *       Input  : fCenOld_y :  
 *       Input  : fThresh   : Threshold for determining center
 *       Output : fCenNew_x :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.1.0 : 200915 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

void ac_radius1(const float *pix_val, const int *mask_val,
             const int iNx, const int iNy,
             const float fRstat,
             const float fCen_x, const float fCen_y,
             const float fThresh,
             float *fRadius)
{  

   int     i,j,k;
   float   fDist;
   float   *fDist_array;
   int     iNdist;
   float   fScale;
   float   fR_max;

   int     iNpix;

#ifdef DEBUG_AC_RADIUS1
   printf("ac_radius1 : Determine radius of a hole with ac_center1 in r=%6.2f pix region \n", fRstat);
#endif

   fDist_array = (float *)malloc(sizeof(float)*(iNx * iNy));

   fScale = 1.0;

   iNdist = 0;
   for (j=1; j<=iNy; j++){
      for (i=1; i<=iNx; i++){
         k = (j-1)*iNx + (i-1); /* Fortran convention */
  
         if (*(pix_val+k) > fThresh && *(mask_val+k)!=MASK_VALUE ){

            fDist = sqrt((i*fScale - fCen_x + 0.5)*(i*fScale - fCen_x + 0.5)
                     +(j*fScale - fCen_y + 0.5)*(j*fScale - fCen_y + 0.5));

            if ( fDist <= fRstat ){
                iNdist = iNdist + 1;
                *(fDist_array+iNdist) = fDist;
            }
          }
       }
    }


    *fRadius = percentile( fDist_array, iNdist, PERCENT_RADIUS );

#ifdef DEBUG_AC_RADIUS1
    fprintf(stderr, "ac_radius1 : New hole radius : fRhole = %f \n", 
           *fRadius );
#endif

    free( fDist_array );
    return;
}
