/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Fit SKY Value
 *
 *       Input  : *pix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

int ac_starview(const float *pix_val, const int *mask_val,
               const int ndatax, const int ndatay,
               const float xcenter_hole, const float ycenter_hole,
               float *xcenter_star, float *ycenter_star,
               const float fRhole, const float fRstar,
               const char *cLine,
               float *ave_sky, float *sig_sky)
{
   int  i,j,k;
   float sum,average,area;
   float fRegion;
   float dis;

   float fmin, fmax;

   static float tr[6] = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0};

   char end;
   float x1,y1;

   float change;
   int   n_pix;
   float *data, sigma_rej, ave_image, sig_image;

   int   n_sky_data;
   float *sky_data, sigma_rej_sky;

   char  cText[100];

   fRegion = 3.0 * fRhole;

   data = (float *)malloc(sizeof(float)*(ndatax * ndatay));
   sky_data = (float *)malloc(sizeof(float)*(ndatax * ndatay));

  n_pix = -1;
  for (i=0; i< ndatax*ndatay -1; i++)
      {
      *(data+i) = 0;
      }

  fmin = fmax = *pix_val ;

   for (j=1; j<=ndatay; j++){
      for (i=1; i<=ndatax; i++){
          k = (j-1)*ndatax + (i-1); /* Fortran convention */
          if ( *(mask_val+k)!=MASK_VALUE ){
             if (*(pix_val+k) < fmin) fmin = *(pix_val+k);
             if (*(pix_val+k) > fmax) fmax = *(pix_val+k);

             n_pix = n_pix + 1;
             *(data + n_pix) = *(pix_val + k);
          }
      }
   }

   sigma_rej = 0.0;

   ac_sigclip(data, n_pix, sigma_rej, 0, &ave_image, &sig_image);

   free(data);

   fmin = *ave_sky - 0 * *sig_sky;

   k = (int) (*ycenter_star+1) * ndatax + (int) (*xcenter_star+1);
   fmax = *(pix_val + k) ;
   fmax = 1.0 * (fmax - fmin) + fmax ;

   end = 'n';


   cpgslw(2);

   while(end!=88 && end!=65){
      end = 'n';

      /* Clear the screen. Set up window and viewport. */
      cpgpage();

      cpgsfs(2);
      cpgsvp(0.1, 0.7, 0.1, 0.1+0.6*1.3);
      cpgswin(1,ndatax,1,ndatay);
      cpggray(pix_val, ndatax, ndatay, 1, ndatax, 1, ndatay, fmin , fmax, tr);
      cpgsci(15);
      cpgcirc(xcenter_hole, ycenter_hole, fRhole);
      cpgsci(13);
      cpgcirc(*xcenter_star, *ycenter_star, fRstar);

      cpgsch(1.0);
      cpgsci(1);
      sprintf(cText, "AC_STARVIEW : %s", cLine);
      cpgptxt( ndatax*1.1, ndatay*1.0, 0.0, 0.0, cText );
      sprintf(cText, "Remove the star : Left click");
      cpgptxt( ndatax*1.1, ndatay*0.50, 0.0, 0.0, cText );
      sprintf(cText, "Write and Quit : Right click");
      cpgptxt( ndatax*1.1, ndatay*0.45, 0.0, 0.0, cText );

      cpgcurs(&x1, &y1, &end);

      sum = 0;
      average = 0;
      area = 0;

      n_sky_data = 0;

      for (j=1; j<=ndatay; j++){
          for (i=1; i<=ndatax; i++){
              k = (j-1)*ndatax + (i-1); 

              if ( *(pix_val+k) >= -1000 ){
                  dis = sqrt((i-xcenter_hole+0.5)*(i-xcenter_hole+0.5)
                                  +(j-ycenter_hole+0.5)*(j-ycenter_hole+0.5));
                  if (dis <= fRhole ){
                         n_sky_data = n_sky_data +1;
                          *(sky_data + n_sky_data - 1) = *(pix_val+k);
                  }
              }
          }
      }

      sigma_rej_sky = 3.0;

      ac_sigclip(sky_data, n_sky_data, sigma_rej_sky, 0, ave_sky, sig_sky);
  }

  if(end==65){
     *xcenter_star = 0.0;
     *ycenter_star = 0.0;
  }


  return EXIT_SUCCESS;

  free(data);
  free(sky_data);
}
