/*********************************************************
 * MOIRCS Acquisition:
 *
 *   Header File
 *
 *   Input : None : None
 *   Output: None : None
 *
 *       Ver.1.1 : 20061112 : Masayuki Akiyama
 *       Ver.1.0 : 20060510 : Masayuki Akiyama
 **********************************************************
 */

#define EXIT_FAILURE 1
#define EXIT_SUCCESS 0

//#define ALIGN_STAR_TAG	"Alignment" // Alignment star tag in mdp file
#define ALIGN_STAR_TAG	"C," // Alignment star tag in sbr file (C,)
#define PLATE_MOIRCS      0.1170 // MOIRCS PLATE SCALE in arcsec/pix
#define PLATE_FOCAS       0.1038 // FOCAS PLATE SCALE in arcsec/pix
#define XLEN_FOCAS          4096 // X AXIS LENGTH OF mosaiced FOCAS IMAGE in pix
#define YLEN_FOCAS          4096 // Y AXIS LENGTH OF mosaiced FOCAS IMAGE in pix
#define XLEN_MOIRCS         2121 // X AXIS LENGTH OF mosaiced MOIRCS IMAGE in pix
#define YLEN_MOIRCS         3641 // Y AXIS LENGTH OF mosaiced MOIRCS IMAGE in pix
#define DEF_RHOLE_MOIRCS      34 // Default hole radius in pix (34)
#define DEF_RBIGHOLE_MOIRCS   85 // Default hole radius in pix (85)
#define DEF_RPINHOLE_MOIRCS    5 // Default pinhole radius in pix (5)
#define DEF_STAR_MOIRCS        4 // Default star size in pix (4)
#define MDP_ERR               15 // Typical MDP error in pix (15)
#define SIZE_SUBARRAY          4 // Size of subarray area in fRhole unit (4)
#define STAT_AREA              3 // Area used for statistics in fRhole unit
#define STAT_AREA_CEN          1.5 // Area used for statistics in fRhole unit
#define STAT_AREA_RAD          1.1 // Area used for statistics in fRhole unit
#define BACKGROUND_THRESH_STAR 3 // Threshold for backgroud area rejection *rms
#define BACKGROUND_THRESH_HOLE 10 // Threshold for backgroud area rejection *rms
#define VIEW_UPSIGMA          10 // Display upper sigma
#define VIEW_LOWSIGMA          2 // Display lower sigma
#define REJECTION_SIGMA      3.0 // Rejection sigma for sigma clipping
#define PERCENT_RADIUS      0.10 // Percentile for radius calculation
#define MASK_EDGE	    0.85 // Mask region of starhole
#define MASK_VALUE             1 // MASK_VALUE
#define MASKED_VALUE	 -1000.0 // value assigned to masked pixel
#define MINNPIX               30 // Minimum number of required pixel for statistics
#define SBRROT                 -0.0 //

#include "cpgplot.h"
#include <fitsio.h>
#include <longnam.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <malloc.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Centaring
 *
 *       Input  : *pix_val  :
 *       Input  : iNx       : x-dimension of pix_val
 *       Input  : iNy       : y-dimention of pix_val
 *       Input  : fFwhm     :
 *       Input  : fCenOld_x :
 *       Input  : fCenOld_y :
 *       Input  : fThresh   :
 *       Output : fCenNew_x :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
void ac_center1(const float *pix_val, const int *mask_val,
             const int iNx, const int iNy,
             const float fRcen,
             const float fCenOld_x, const float fCenOld_y,
             const float fThresh,
             float *fCenNew_x, float *fCenNew_y);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Centaring
 *
 *       Input  : *pix_val  :
 *       Input  : iNx       : x-dimension of pix_val
 *       Input  : iNy       : y-dimention of pix_val
 *       Input  : fFwhm     :
 *       Input  : fCenOld_x :
 *       Input  : fCenOld_y :
 *       Input  : fThresh   :
 *       Output : fCenNew_x :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
void ac_center2(const float *pix_val, const int *mask_val,
             const int iNx, const int iNy,
             const float fRcen,
             const float fCenOld_x, const float fCenOld_y,
             const float fThresh,
             float *fCenNew_x, float *fCenNew_y);

/*********************************************************
 * MOIRCS acquisition:
 *
 *    Determin radius of a hole using pixel value above fThresh
 *       < equal weight, not centroid >
 *
 *       Input  : *pix_val  :
 *       Input  : iNx       : x-dimension of pix_val
 *       Input  : iNy       : y-dimention of pix_val
 *       Input  : fRstat    : Radius of region used for statistics
 *       Input  : fCenOld_x :
 *       Input  : fCenOld_y :
 *       Input  : fThresh   : Threshold for determining center
 *       Output : fCenNew_x :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.1.0 : 200915 : Masayuki Akiyama
 **********************************************************
 */
void ac_radius1(const float *pix_val, const int *mask_val,
             const int iNx, const int iNy,
             const float fRstat,
             const float fCen_x, const float fCen_y,
             const float fThresh,
             float *fRadius);

/*********************************************************
 *  MOIRCS Acquisition:
 *
 *    Pick up Hole manually
 *
 *       Input  : *pix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.0.0 : 20050123 : Masayuki Akiyama
 **********************************************************
 */
int ac_pickhole(float *pix_val,
   const int *axlen, const char *cLine,
   const int iNhole, float *pHoleX, float *pHoleY);

int ac_pickstar(float *pix_val,
   const int *axlen, const char *cLine,
   const int iNhole, float *pHoleX, float *pHoleY);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Determin MODE
 *
 *       Input  : *pData    : POINT Data
 *       Input  : dNdata    : INT   Number of Data
 *       Output : fAverage  : FLOAT Average
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
float ac_mode(const float *data, const int ndata);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Calcurate Average W/O Any Rejection
 *
 *       Input  : *pData    : POINT Data
 *       Input  : dNdata    : INT   Number of Data
 *       Output : fAverage  : FLOAT Average
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
float ac_average(const float *pData, const int dNdata);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Sigma Clipping
 *
 *       Input  : *pData    : Data
 *       Input  : iNdata    : Number of Data
 *       Input  : fRejsig   : Rejection Sigma
 *       Input  : dMode     : AVERAGE:0 MODE:1
 *       Output : *fMean    : Mean Value
 *       Output : *fSigma   : RMS
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
void ac_sigclip(const float *pData, const int iNdata,
                const float fRejsig, const int dMode,
                  float *fMean, float *fSigma);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Fit SKY Value
 *
 *       Input  : *pix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
int ac_holeview(const float *pix_val, const int *mask_val,
               const int ndatax, const int ndatay,
               float *xcenter, float *ycenter,
               const float fwhm,
               const char *cLine,
               float *ave_sky, float *sig_sky);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Fit SKY Value
 *
 *       Input  : *pix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
int ac_starview(const float *pix_val, const int *mask_val,
               const int ndatax, const int ndatay,
               const float xcenter_hole, const float ycenter_hole,
               float *xcenter_star, float *ycenter_star,
               const float fRhole, const float fRstar,
               const char *cLine,
               float *ave_sky, float *sig_sky);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Correct Bad Pixel Region
 *
 *       Input  : *pix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
int ac_corre_img0(float *pix_val, int *mask_val,
   const int ndatax, const int ndatay,
   const float xcenter, const float ycenter,
   const char *cLine, const int iInterac);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Correct Bad Pixel Region
 *
 *       Input  : *pix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
int ac_corre_img1(float *pix_val, int *mask_val,
   const int ndatax, const int ndatay,
   const float xcenter, const float ycenter,
   const char *cLine, const int iInterac);

/*********************************************************
 * MOIRCS Acquisition:
 *
 *    Correct Bad Pixel Region
 *
 *       Input  : *pix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
int ac_corre_img2(float *pix_val, int *mask_val,
   const int iNx, const int iNy,
   const float fCen_X, const float fCen_Y, const float fRadius,
   const char *cLine, const int iInterac);

/*********************************************************
 * MOIRCS acquisition:
 *
 *     Determine Threshold using pixel value
 *
 *       Input  : *pix_val  :
 *       Input  : iNx       : x-dimension of pix_val
 *       Input  : iNy       : y-dimention of pix_val
 *       Input  : fRcen     :
 *       Input  : fCen_x    :
 *       Input  : fCen_y    :
 *       Output : fThresh   : Threshold for determining center
 *
 *
 *       Ver.0.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
void ac_thresh1(const float *pPix_org, const int *pMask,
             const int iNx, const int iNy,
             const float fRcen,
             const float fCen_x, const float fCen_y,
             const float fThresh_fact,
             float *fThresh);

/*********************************************************
 * MOIRCS acquisition:
 *
 *     Determine Threshold using pixel value
 *
 *       Input  : *pix_val  :
 *       Input  : iNx       : x-dimension of pix_val
 *       Input  : iNy       : y-dimention of pix_val
 *       Input  : fRhole    :
 *       Input  : fCen_x    :
 *       Input  : fCen_y    :
 *       Output : fThresh   : Threshold for determining center
 *
 *
 *       Ver.0.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */
void ac_thresh2(const float *pPix_org, const int *pMask,
             const int iNx, const int iNy,
             const float fRhole,
             const float fCen_x, const float fCen_y,
             float *fThresh);

/*****************************************************
 *  MOIRCS Acquisition:
 *
 *    Read FITS Image Size
 *
 *   Input : cFilename : FITS file name
 *   Output: pAxis     : Length of axis
 *   Output:           : Status from cfitsio
 *
 *     Version 2.0 20040305 Masayuki Akiyama from MMT
 *****************************************************
 */
int fits_read_size(const char *cFilename, int *iAxis);

/*****************************************************
 *  MOIRCS Acquisition:
 *
 *    Read Subarray of a FITS image
 *
 *   Input : cFilename : FITS filename
 *   Input : pRange    : XY range (xmin, xmax, ymin, ymax)
 *   Output: pArray    : Sub array image
 *   Output:           : Return status from cfitsio
 *
 *     Version 2.0 20030830 Masayuki Akiyama from MMT
 *****************************************************
 */
int fits_read_subarray(const char *cFilename, const int *pRange, float *pArray);

/*****************************************************
 *  MOIRCS Acquisition:
 *
 *    Read FITS Header Keyword
 *
 *   Input : cFilename : FITS file name
 *   Output: pAxis     : Length of axis
 *   Output:           : Status from cfitsio
 *
 *     Version 2.0 20040305 Masayuki Akiyama from MMT
 *****************************************************
 */
int fits_read_head(const char *cFilename, const char *cKeyname,
               char *cRecord);

/*****************************************************
 *  MOIRCS Acquisition:
 *
 * Count Number of Data in Datafile
 *
 *   Input : cFname : Input File Name
 *   Output: (int)  : Number of Data in the Input File
 *
 *     Version 0.0 20050123 Masayuki Akiyama
 *****************************************************
 */
int file_count_data(const char *cFname);

/*****************************************************
 *  MOIRCS Acquisition:
 *
 *   Read Data File
 *
 *   Input : cFile   : Input Data File Name
 *   Output: pObject : Object Data Struct
 *   Output:         : Read Data Array
 *
 *
 *     Version 0.0 20050123 Masayuki Akiyama
 *****************************************************
 */
int file_read_data(const char *cFile, float *fHoleX, float *fHoleY);

/*****************************************************
 *  MOIRCS Acquisition:
 *
 *   Read Data File
 *
 *   Input : cFile   : Input Data File Name
 *   Output: fHoleX  : X coordinate list
 *   Output: fHoleY  : Y coordinate list
 *   Output: iStat   : <0 if iC=0, or if fileread err
 *                   : =iC : Number of data read
 *
 *     Version 1.0 20050915 Masayuki Akiyama
 *****************************************************
 */
int file_read_mdp(const char *cFile, float *fHoleX, float *fHoleY, const char *cType);
int file_read_sbr(const char *cFile, float *fHoleX, float *fHoleY, const char *cType);

/*****************************************************
 *  MOIRCS Acquisition:
 *
 *   Read Data File
 *
 *   Input : cFile   : Input Data File Name
 *   Output: fHoleX  : X coordinate list
 *   Output: fHoleY  : Y coordinate list
 *   Output: iStat   : <0 if iC=0, or if fileread err
 *                   : =iC : Number of data read
 *
 *     Version 1.0 20050915 Masayuki Akiyama
 *****************************************************
 */
int file_read_hole(const char *cFile, float *fHoleX, float *fHoleY, float *fHoleR);

/*********************************************************
 * MOIRCS acquisition:
 *
 *    Calculate percentile
 *
 *       Input  : pData     : Array of data
 *       Input  : iNdata    : Number of data
 *       Output :  :
 *
 *       Ver.1.0 : 20050915 : Masayuki Akiyama
 **********************************************************
 */
float percentile(const float *pData, const int iNdata, const float fPercent);

/*********************************************************
 *  MOIRCS Acquisition:
 *
 *   Apply Gaussian filter to the image data array
 *
 *       Input  : *fPix_val   :
 *       Input  : *iAxlen     : Image XY axis length
 *       Input  : fSigma      :
 *       Output : *fpix_val_g :
 *
 *       Ver.0.0 : 20050123 : Masayuki Akiyama
 **********************************************************
 */
int ac_gaussfilter(const float *fPix_val, const int *iAxlen,
   const float fSigma, float *fPix_val_g );

