/*****************************************************
 *  Aperture Photometory:
 *
 *    Read FITS Image Size
 *
 *   Input : cFilename : FITS file name
 *   Output: pAxis     : Length of axis
 *   Output:           : Status from cfitsio
 *
 *     Version 0.0 20040305 Masayuki Akiyama from MMT
 *****************************************************
 */

#include  "acquisition.h"

#undef DEBUG
 
int fits_read_size(const char *cFilename, int *iAxis)
{
   fitsfile  *pFits;
   long      iAxlen[2]={1024, 1024};
   int       iStat;
   int       iNfound;

   iStat = 0;

   if(fits_open_file(&pFits, cFilename, READONLY, &iStat)>0){
         fits_report_error(stderr, iStat);
         exit(8);
   }
   if(fits_read_keys_lng(pFits, "NAXIS", 1, 2, iAxlen, &iNfound, &iStat)>0){
         fits_report_error(stderr, iStat);
         exit(8);
   }
   *(iAxis+0) = iAxlen[0];
   *(iAxis+1) = iAxlen[1];

   if(fits_close_file(pFits, &iStat)){ 
         fits_report_error(stderr, iStat);
         exit(8);
   }

   return iStat;
}
