# mcsgeocorr.cl ... created by IT 2006-01-29
# fully-revised version of mcsdistcrr.cl
# distortion files and mask file names are all read from the mcsred 
#   configuration files.
#
# !!! This task is available for the following periods. !!!
#	Jan 2005	config=dir_mcsred$DATABASE/ana_jan05.cfg
#	Dec 2005	config=dir_mcsred$DATABASE/ana_dec05.cfg
#	Jan-Feb 2005	config=dir_mcsred$DATABASE/ana_jan06.cfg
#
# Last Revise: 2006-04-03:
# 	Changed the format of the input file.


procedure mcsgeocorr (infile)

string 	infile 		{prompt="Input image name (allow @list format)"}
string	config	{"dir_mcsred$DATABASE/ana_mar06.cfg", prompt="the name of mcsred config file"}
bool 	crrej   	{no, prompt="Execute cosmicray cleaning? (time consuming...)"}

begin
	int fnd, detid, j
	real sky1, sig1
	string s1, s2, s3, msk
	string dbs[2], gmp[2], gcm[2]
	string in, out, hd

	
	if (access("tmp_in.lst"))
		del ("tmp_in.lst", ver-)
	
	if (access("tmp1.fits"))
		imdel ("tmp1.fits", ver-)

	if (access ("tmpgc.lst"))
		del ("tmpgc.lst", ver-)
	
# reading mcsred configuration file	
	list=config
	while (fscan (list, s1, s2) !=EOF){
		s3=substr(s1, 1, 1)
		if (s3 != "#"){
			if (s1 == "3")
				dbs[1]=s2
			else if (s1 == "4")
				gmp[1]=s2
			else if (s1=="5")
			    dbs[2]=s2
			else if (s1 == "6")
				gmp[2]=s2
			else if (s1 == "7")
				gcm[1]=s2
			else if (s1 == "8")
				gcm[2]=s2
		}
	}
#

	j=strlen(infile)
    hd=substr(infile,1,1)

    if (hd != "@"){
        print (infile, >> "tmp_in.lst")
        list="tmp_in.lst"
    }
    else{
        infile=substr(infile, 2, j)
        list=infile
        if (access("d"//infile)){
            print ("The file ", "d"//infile, " exists. Delete or rename it.")
		}
	}
	
	while (fscan(list, s1) !=EOF){

		basename (s1) | scan (s2)
		if (!access(s2//".fits"))
			bye
		
		hselect (s1, "DET-ID", expr+) | scan(detid)
		print ("CHECK: image= ", s2, "  detector=", detid)
		print ("CHECK: distmask= ", gcm[detid], "  database=", dbs[detid]) 
	
# comic-ray cleaning:
		if (crrej){
			hselect (s1, "BPM", expr+) | scan (msk)
			if (access(msk)){
				print ( "cosmic-ray cleaning. Mask=", msk )
				imstat (s1//"[800:1300,800:1300]", field="midpt,stddev",  nclip=3, form-) | scan (sky1, sig1)
				imcopy (s1, "tmp1.fits", ver-)
				imreplace ("tmp1.fits", sky1, upper=sky1-5*sig1)
				fixpix ("tmp1.fits", "BPM")
				cosmicrays ("tmp1.fits", "tmp1.fits", threshold=5*sig1, fluxrat=10, npass=10, inter-)
			}
		else
			print ("No associated mask file", msk,". Ignore crrej.")
		}
## 

		if (crrej){
			geotran ("tmp1.fits", "gc"//s1, dbs[detid], gmp[detid], boundary="constant", constant=0)
			imdel ("tmp1.fits", ver-)
		}
		else {
			geotran (s1, "gc"//s1, dbs[detid], gmp[detid], boundary="constant", constant=0)
		}
		hedit ("gc"//s1, "BPM", gcm[detid], add+, update+, ver-)

		print ("Distortion Correction for ", s1, " is finished!")

		print ("gc"//s1, >> "tmpgc.lst")
		
	}

	if (hd == "@"){
#	cp ("tmpgc.lst", "gc"//infile)
#	print ("The list of output images --> ", "gc"//infile)
		mv ("tmpgc.lst", "d"//infile)
		print ("The list of output images --> ", "d"//infile)
	}	
	else {
		del ("tmpgc.lst", ver-)
		del ("tmp_in.lst", ver-)
	}
	
end


