/*********************************************************
 * MOIRCS Acquisition: 
 *
 *     mes_hole
 *
 *   Input : hole_radius   : Hole radius in pixel
 *   Input : hole_pos_list : Hole position file name (x y) text
 *   Input : outputfile    : Output filename
 *
 *       Ver.1.0 : 20050915 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

#undef DEBUG

int main( argc, argv )
int     argc;
char    *argv[];
{
   int    i;
   int    dNx, dNy;
   int    axlen[2];
   int    *pRange0;
   float  *pix_val; 

   float  fXshift, fYshift;
   float  fRhole;
   float  fRhole_new;
   float  fSky_ave, fSky_sig;

   int    iNite, iNite_max;
   float  fXcenter_stat, fYcenter_stat;
   float  fR_stat_cen; /* Centering Radius */
   float  fR_stat_rad; /* Centering Radius */
        
   int    iNdatax_cut, iNdatay_cut;
   float  *pix_val_cut; 
   int    *mask_val_cut;
   int    iNhole;
   int    iPix;
   float  *pHoleX, *pHoleY;

   float  fXcenter_cut, fYcenter_cut;
   float  fXcenter_new, fYcenter_new;

   float  fThresh;

   int   iCheck;
   int   iInterac;

   FILE   *pFile0;  
   char   cLine[1000];

   if(argc != 5){
      printf("usage : test [image] [hole_pos_list(X Y)] [outputfile] [interac?]\n");
      printf("  hole_pos_list : X Y \n");
      printf("  hole_pos_list SHOULD BE .sbr file in FUTURE \n"); 
      exit(1);
   }
   fRhole = DEF_RHOLE_MOIRCS;
     
/*
 * Read Image Data
 */
   fits_read_size( argv[1], axlen );
#ifdef DEBUG_MES_HOLE
   printf("%s Image size = %d %d \n", argv[1], axlen[0], axlen[1]);
#endif

   pix_val = (float *)malloc(sizeof(float)*(axlen[0] * axlen[1]));
  
   pRange0 = (int *)malloc(sizeof(int)*4);

   *(pRange0+0) = 0;
   *(pRange0+1) = axlen[0]-1;
   *(pRange0+2) = 0;
   *(pRange0+3) = axlen[1]-1;

   if(fits_read_subarray( argv[1], pRange0, pix_val ) < 0){
       exit(0);
   }

/*
 * STEP 0: Read Hole List
 */
    iNhole = file_count_data(argv[2]);
    pHoleX = (float *)malloc(sizeof(float) * iNhole);
    pHoleY = (float *)malloc(sizeof(float) * iNhole);
//    iNhole = file_read_mdp(argv[2], pHoleX, pHoleY, ALIGN_STAR_TAG);
    iNhole = file_read_sbr(argv[2], pHoleX, pHoleY, ALIGN_STAR_TAG);

#ifdef DEBUG_MES_HOLE
    fprintf( stderr, "Number of holes in the SBR file %s = %d \n", argv[2], iNhole);
#endif

/*
 * STEP 1: Pick 1st hole and 
 *       :    shift coordinates of the holes with the measured position of the 1st hole
 */

    if(cpgbeg(0, "/xw", 1, 1) != 1){
         exit(EXIT_FAILURE);
    }
    cpgask(0); 
    cpgpage(); 
    cpgbbuf(); 

    dNx = axlen[0];
    dNy = axlen[1];
    ac_pickhole(pix_val, axlen, cLine, iNhole, pHoleX, pHoleY );

    cpgebuf(); 

// Open recored file
    pFile0 = fopen(argv[3],"at");  

    for(i=0; i<iNhole; i++){

/*
 * STEP 2 : Extract Subarray around a HOLE
 */

       iCheck = 0;

       *(pRange0+0) = (int) (*(pHoleX+i) - SIZE_SUBARRAY/2.0 * fRhole);
       *(pRange0+1) = (int) (*(pHoleX+i) + SIZE_SUBARRAY/2.0 * fRhole);
       *(pRange0+2) = (int) (*(pHoleY+i) - SIZE_SUBARRAY/2.0 * fRhole); 
       *(pRange0+3) = (int) (*(pHoleY+i) + SIZE_SUBARRAY/2.0 * fRhole);

       if( *(pRange0+0) < 1 || *(pRange0+0) >= axlen[0] ){ iCheck=1; }
       if( *(pRange0+1) < 1 || *(pRange0+1) >= axlen[0] ){ iCheck=1; }
       if( *(pRange0+2) < 1 || *(pRange0+2) >= axlen[1] ){ iCheck=1; }
       if( *(pRange0+3) < 1 || *(pRange0+3) >= axlen[1] ){ iCheck=1; }

       if( iCheck == 1){
           fprintf(stderr,"mes_hole : The hole is outside of the image\n");
           fprintf(pFile0, "    0.0     0.0     0.0 \n");
       }
       else{
           iNdatax_cut = *(pRange0+1) - *(pRange0+0) + 1;
           iNdatay_cut = *(pRange0+3) - *(pRange0+2) + 1;

           pix_val_cut = (float *)malloc(sizeof(float)*(iNdatax_cut*iNdatay_cut));
           mask_val_cut = (int *)malloc(sizeof(int)*(iNdatax_cut*iNdatay_cut));

           if(fits_read_subarray( argv[1], pRange0, pix_val_cut ) < 0){
               exit(0);
           }

           for(iPix=0; iPix<(iNdatax_cut*iNdatay_cut); iPix++){
               *(mask_val_cut+iPix) = 0;
           }

           fXcenter_cut = *(pHoleX+i) - ( *(pRange0+0) - 0.5 );
           fYcenter_cut = *(pHoleY+i) - ( *(pRange0+2) - 0.5 );

           cpgpage(); 
           cpgbbuf(); 

           sprintf( cLine, "HOLE %d ", i+1 );
        
/*
 * STEP 3 : Remove Bad Pixel etc.
 */
           iInterac = 1;
           if( strcmp(argv[4], "no")==0 ){
              iInterac = 0;
           }
           ac_corre_img1(pix_val_cut, mask_val_cut, iNdatax_cut,iNdatay_cut,
                     fXcenter_cut,fYcenter_cut, cLine, iInterac); 
 
           cpgebuf(); /* end batch of output */   


/*
 * Repeat Step 4 & 5
 */
           iNite_max = 10;
           fR_stat_cen = STAT_AREA_CEN * fRhole;
           fR_stat_rad = STAT_AREA_RAD * fRhole;
           fXcenter_stat = fXcenter_cut;
           fYcenter_stat = fYcenter_cut;

           for( iNite=0; iNite<iNite_max; iNite++ ){

#ifdef DEBUG_MES_HOLE
    fprintf( stderr, " mes_hole : Determine background level threshold  \n");
#endif
              ac_thresh1(pix_val_cut, mask_val_cut, iNdatax_cut,iNdatay_cut,
                      fR_stat_cen, fXcenter_stat, fYcenter_stat, BACKGROUND_THRESH_HOLE, &fThresh);
   
#ifdef DEBUG_MES_HOLE
    fprintf( stderr, " mes_hole : Determine centroid of the hole within %f pix area  \n", fR_stat_cen);
#endif
      
              ac_center1(pix_val_cut, mask_val_cut, iNdatax_cut, iNdatay_cut,
                    fR_stat_cen, fXcenter_stat, fYcenter_stat, fThresh,
                    &fXcenter_new, &fYcenter_new); 

#ifdef DEBUG_MES_HOLE
              fprintf(stderr, "Original center = %f %f \n", fXcenter_cut, fYcenter_cut);
              fprintf(stderr, "Revised center = %f %f \n", fXcenter_stat, fYcenter_stat);
              fprintf(stderr, "New center = %f %f \n", fXcenter_new, fYcenter_new);
#endif

              fXcenter_stat = fXcenter_new;
              fYcenter_stat = fYcenter_new;

#ifdef DEBUG_MES_HOLE
    fprintf( stderr, "Determine radius of the hole within %f pix area  \n", fR_stat_rad);
#endif
              ac_radius1(pix_val_cut, mask_val_cut, iNdatax_cut, iNdatay_cut,
                    fR_stat_rad, fXcenter_stat, fYcenter_stat, fThresh,
                    &fRhole_new); 

              fR_stat_cen = STAT_AREA_CEN * fRhole_new;
              fR_stat_rad = STAT_AREA_RAD * fRhole_new;
           }

#ifdef DEBUG_MES_HOLE
    fprintf( stderr, "Show image \n");
#endif

           cpgpage(); 
           cpgbbuf(); 
       
           if( ac_holeview(pix_val_cut,mask_val_cut,iNdatax_cut,iNdatay_cut,
                   &fXcenter_new, &fYcenter_new, fRhole_new,
                   cLine,
                   &fSky_ave, &fSky_sig) > 0 ){
             exit(0);
           }
 
           cpgebuf(); /* end batch of output */   

/*
 * STEP 7 : Record Measured Data
 */
           if( fXcenter_new > 0 && fYcenter_new > 0 ){
              fprintf(pFile0, "%7.1f %7.1f %7.1f \n", 
              fXcenter_new +(*(pRange0+0)-0.5), fYcenter_new +(*(pRange0+2)-0.5),
              fRhole_new);
           }
           else{
              fprintf(pFile0, "    0.0     0.0     0.0 \n");
           }
       }
    }
    fclose(pFile0);  

/*
 * STEP 7 : End PGPLOT
 */
           cpgend(); 

/*
 * Free Used Memory
 */
	free(pix_val);
	free(pix_val_cut);
	free(mask_val_cut);
        free(pHoleX);
        free(pHoleY);

  return EXIT_SUCCESS;
}
