"""
mdparith.py - MDP spectroscopic data arithmetiric
2007.07.23. YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
"""
import os
from pyraf import iraf
import numpy
from iraftool import *
from spectool import *

def mdparith(op1, op, op2, result,
             key_op1_wl='WAVELENGTH',key_op1_fl='FLUX',key_op2_wl='WAVELENGTH',key_op2_fl='FLUX',
             filter=False
             ):
    """
    calculate opfile (2D or 1D spec FITS or 1D bin table FITS) op([+-*/]) op2
    op2 shold be MCSMDP-type 1D bin table FITS
    key_op[12]_wl: key name of wavelength for op[12] (1D bin table only)
    key_op[12]_fl: key name of flux for op[12] (1D bin table only)
    Note: op2 is interpolated to op1
    """
    
    list1 = fileutil.fileparse(op1, check=True, ext=True)
    list2 = fileutil.fileparse(op2, check=True, ext=True)
    listr = fileutil.fileparse(result, ext=True)
    if len(list2) > 0:
        l2_idx = range(len(list2))
    else:
        l2_idx = numpy.zeros(len(list1))
    
    for file1, idx2, filer in zip(list1, l2_idx, listr):
        fname1 = fileutil.osimg(file1[0])
        ext1 = file1[1]
        f1 = pyfits.open(fname1)
        fname2 = fileutil.osimg(list2[idx2][0])
        ext2 = list2[idx2][1]
        f2 = pyfits.open(fname2)

        if ext1 > 0:
            if f1[ext1].header['XTENSION'] == 'BINTABLE':
                resd = specdata.spec_arith(f1[ext1].data, op, f2[ext2].data,
                                           key_op1_w1=key_op1_wl, key_op1_fl=key_op1_fl,
                                           key_op2_w1=key_op2_wl, key_op2_fl=key_op2_fl,
                                           filter=filter)
                res = pyfits.BinTableHDU(resd, f1[ext1].header)
            else:
                res = specdata.twodspec_arith(f1[ext1], op, f2[ext2].data,
                                              key_op2_wl=key_op2_wl, key_op2_fl=key_op2_fl,)
        else:
            res = specdata.twodspec_arith(f1[ext1], op, f2[ext2].data,
                                          key_op2_wl=key_op2_wl, key_op2_fl=key_op2_fl,)

        fnamer = fileutil.osimg(filer[0])
        if(os.access(fileutil.osimg(fnamer), os.F_OK)):
            fr = pyfits.open(fnamer, mode='update')
            fr.append(res)
            fr.close()
        else:
            res.writeto(fnamer)

