"""
mdpextract.py - extract 1D spectrum from MDP MEF file
2007.07.31. YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.8  2009/02/08 01:31:17  tomohiro
add bottom plot

Revision 1.7  2008/06/20 04:00:55  tomohiro
add fit_interactive and type_of_fit

Revision 1.6  2008/03/19 10:20:22  tomohiro
modify to accept outfile with different path

Revision 1.5  2008/02/01 23:26:15  tomohiro
modify to accept BINTABLE fits

Revision 1.4  2007/12/20 07:20:40  tomohiro
add options for [OIII]/Hb

Revision 1.3  2007/10/07 05:40:51  tomohiro
add implot for mdp2dplot()

Revision 1.2  2007/09/26 11:59:18  tomohiro
add  plotsn, title, xlabel and ylabel to mdp2dplot()
add add title, xlabel, ylabel, lstyle, redshift, linelist, linealpha, linewidth to mdpplot()

Revision 1.1  2007/09/03 08:23:52  tomohiro
extract 1D spectrum from MDP MEF file

"""
import os
from pyraf import iraf
import pyfits,pylab,numpy
from iraftool import *
from spectool import *

iraf.stsdas(_doprint=False)
iraf.contrib(_doprint=False)
iraf.spfitpkg(_doprint=False)

def set_fname(infile, indir=None):
    if infile[0] != '/':
        if indir == None:
            indir = os.getcwd()
        outfile = indir + '/' + infile
    else:
        outfile = infile
    return outfile

def mdp2dplot(infile, line, center, nsum=5, width=1000, bgmin=10, bgmax=30, noise='noise',
              style='k-', errorbar=True, xmin=iraf.INDEF, xmax=iraf.INDEF,
              ymin=iraf.INDEF, ymax=iraf.INDEF, yscale=1,
              title='INDEF', xlabel='INDEF', ylabel='INDEF', boxcar=iraf.INDEF, bin=iraf.INDEF,
              plotoh=True, plotres=True, ploterror=True, plotfit=False, plotsn=False,
              ohstyle='k-', rstyle='k--', estyle='k:', lstyle='k-', fstyle='k--',
              ohfile='self', rescurve='INDEF', sfdbfile='INDEF', grid=0, legend=False,
              redshift=0, restwl=True, linelist=None, linealpha=0.2, linewidth=10,
              implot=False, imywidth=20, cnt=4, outfile='INDEF',
              bottomplot=True,
              ):
    
    infile = fileutil.osimg(infile)
    indir = os.path.split(infile)[0]
    print indir
    if xmin == iraf.INDEF: xmin = None
    if xmax == iraf.INDEF: xmax = None
    if ymin == iraf.INDEF: ymin = None
    if ymax == iraf.INDEF: ymax = None
    if boxcar == iraf.INDEF: boxcar = None
    if bin == iraf.INDEF: bin = None
    if grid == iraf.INDEF: grid = None
    if linelist == 'INDEF': linelist = None
    if title == 'INDEF': title = None
    if xlabel == 'INDEF': xlabel = None
    if ylabel == 'INDEF': ylabel = None
    
    inhead = pyfits.getheader(infile, ext=1)
    if inhead['XTENSION'] == 'BINTABLE':
        line = inhead['MDP_SUMC']
        sfdbfile = set_fname(inhead['MDP_SFDB'])
        infile = set_fname(inhead['MDP_IMSP'])
        nsum = inhead['MDP_NSUM']
        if inhead['MDP_SFTP'] == 'Halpha':
            wl = sfdb.open(inhead['MDP_SFDB'])['gaussian1'][1][0]
            redshift = wl/6563 - 1
            if center < 0:
                center = wl
        elif inhead['MDP_SFTP'] == 'OIII':
            wl = sfdb.open(inhead['MDP_SFDB'])['gaussian1'][1][0]
            redshift = wl/5007 - 1
            if center < 0:
                center = sfdb.open(inhead['MDP_SFDB'])['gaussian2'][1][0]
    
    data, head = twodspec.onedextract(infile, line, center, outfile=None, header=True,
                                      nsum=nsum, width=width, bgmin=bgmin, bgmax=bgmax, noise=noise)
    if plotoh:
        ohfile = iraf.osfn(ohfile)
    else:
        ohfile = None

    if rescurve == 'INDEF' and head.has_key('MDP_RESP'):
        rescurve = set_fname(head['MDP_RESP'], indir)
    if plotres and rescurve == 'INDEF':
        print 'Warning: response curve database file is undefined'
    else:
        rescurve = None
    if sfdbfile == 'INDEF' and head.has_key('MDP_SFDB'):
        sfdbfile = set_fname(head['MDP_SFDB'], indir)
    if plotfit and sfdbfile == 'INDEF':
        print 'Warning: specfit database file is undefined'
    elif plotfit:
        fitfunc = sfdb.open(sfdbfile).get_func()
    else:
        fitfunc = None
    if implot:
        imspec = infile
        imymin = line - int(imywidth/2.)
        imymax = line + int(imywidth/2.)
    else:
        imspec = None
        imymin = None
        imymax = None
    
    # set title
    if title == None:
        title = "%s [%.1fA:%.1fA][%d:%d]" % (head['OBJECT'],
                                             center-width/2., center+width/2.,
                                             line-int(nsum/2), line+int(nsum/2))
    
    twodspec.onedplot(data, style, errorbar=errorbar, xmin=xmin, xmax=xmax,
                      ymin=ymin, ymax=ymax, yscale=yscale,
                      xlabel=xlabel, ylabel=ylabel, boxcar=boxcar, bin=bin,
                      ohfile=ohfile, rescurve=rescurve, error=ploterror, fitfunc=fitfunc, snplot=plotsn,
                      grid=grid, legend=legend, title=title,
                      ohstyle=ohstyle, rstyle=rstyle, fstyle=fstyle, estyle=estyle,
                      redshift=redshift, restwl=restwl, linelist=linelist,
                      lstyle=lstyle, linealpha=linealpha, linewidth=linewidth,
                      imspec=imspec, imymin=imymin, imymax=imymax, bottomplot=bottomplot)

    if outfile != 'INDEF':
        pylab.savefig(outfile)
    
def mdpextract(infile, line, center, outfile, nsum=5, width=1000, bgmin=10, bgmax=30, noise='noise',
               fittype='Halpha', name='INDEF', fit_interactive=True, type_of_fit='simplex',
               iflux=iraf.INDEF, icenter=iraf.INDEF, ifwhm=iraf.INDEF,
               center_tolerance=100, fix_fwhm=False, cont_flux=0, cont_kmax=4.0e-22,
               sub_feature=True, sub_frac=iraf.INDEF, sub_fwhm=iraf.INDEF, logfile='INDEF',
               o3_4959=True, broad=False
               ):
    infile = fileutil.osimg(infile)
    if pyfits.getheader(infile, ext=1)['XTENSION'] == 'BINTABLE':
        if outfile != 'INDEF':
            iraf.copy(infile, outfile)
        else:
            outfile = infile

    outbase, outdir = fileutil.basename(outfile, suffix=True, dir=True)[0:2]
    tmp_sfdata =  'sf' + outbase + '.dat'
    dbname = fittype + outbase + '.db'
    dbnamef = fittype + outbase + '_final.db'
    tmp_sfdb = outdir + '/sfdb/sf' + dbname
    sfdbf = outdir + '/sfdb/sf' + dbnamef
    dbdir = outdir + '/sfdb'
    outfile = fileutil.osimg(outfile)
    if(not os.access(dbdir, os.F_OK)): os.mkdir(dbdir)
    
    if iflux == iraf.INDEF: iflux = None
    if icenter == iraf.INDEF: icenter = None
    if ifwhm == iraf.INDEF: ifwhm = None
    if sub_frac == iraf.INDEF:
        if fittype == 'Halpha':
            sub_frac = 0.1
        elif fittype == 'OIII':
            sub_frac = 0.5
        else:
            sub_frac = None
    if sub_fwhm == iraf.INDEF: sub_fwhm = None
    
    #print tmp_sfdb
    if pyfits.getheader(infile, ext=1)['XTENSION'] != 'BINTABLE':
        if(os.access(outfile, os.F_OK)): os.unlink(outfile)
        twodspec.onedextract(infile, line, center, outfile=outfile,
                             nsum=nsum, width=width, bgmin=bgmin, bgmax=bgmax, noise=noise)

    #print outfile
        
    if logfile == 'INDEF':
        logfile = outdir + '/specfit'+fileutil.basename(outfile, suffix=True, dir=True)[0]+'.log'
    
    if fittype != 'INDEF':
        sdata = pyfits.getdata(outfile)
        
        specdata.writeto(sdata, tmp_sfdata, format='specfit')
        
        if fittype == 'Halpha' or fittype == 'OIII' or fittype == 'emission':
            emdb = mksfdb.emission(sdata, em_flux=iflux, em_center=icenter, em_fwhm=ifwhm, center_tolerance=center_tolerance,
                                   fix_fwhm=fix_fwhm, cont_flux=cont_flux, cont_kmax=cont_kmax, em_type=fittype,
                                   sub_frac=sub_frac, sub_feature=sub_feature, sub_fwhm=sub_fwhm, name=name,
                                   o3_4959=o3_4959, broad=broad)
            emdb.writeto(tmp_sfdb)
        else:
            raise TypeError, 'invalid fittype: '+fittype
        iraf.specfit(tmp_sfdata, initial_fit=dbname, final_fit=dbnamef, logfile=logfile, database=dbdir,
                     interact=fit_interactive, type_of_fit=type_of_fit, 
                     Sample_ranges='*', max_iterations=100)
        
        outfits = pyfits.open(outfile, mode='update')
        outfits[1].header.update('MDP_SFDB', sfdbf, comment='database file name for specfit')
        outfits[1].header.update('MDP_SFLG', logfile, comment='log file name for specfit')
        outfits[1].header.update('MDP_SFTP', fittype, comment='database file type for specfit')
        outfits.close()
        
        if(os.access(tmp_sfdata, os.F_OK)): os.unlink(tmp_sfdata)
        
        # print results
        sflog = sfdb.SFLOG(logfile)
        print "MDPEXTRACT:",infile
        print "chi-square probability: %.2f %%"%(sflog['P']*100.)
        for key, name in zip(sflog.get_compnames(), sflog.get_comments()):
            comp = sflog.get_component(key)
            print "  %s:"%(name)
            for k, v in comp.data.iteritems():
                if k == 'flux' or k == 'slope':
                    print "    %s: %.2e (+/-%.2e)"%(k, v[0], v[1])
                else:
                    print "    %s: %.2f (+/-%.2f)"%(k, v[0], v[1])
    

def mdpplot(infile, outfile=None, style='k-', errorbar=True,
            xmin=None, xmax=None, ymin=None, ymax=None, yscale=1,
            title='INDEF', xlabel='INDEF', ylabel='INDEF', boxcar=None, bin=None,
            plotoh=True, plotres=True, ploterror=True, plotfit=True,
            ohstyle='k-', rstyle='k--', estyle='k:', lstyle='k-', fstyle='k-.', 
            ohfile='self', rescurve=None, sfdbfile=None,
            grid=0, legend=False,
            redshift=0, restwl=True, linelist=None, linealpha=0.2, linewidth=10,
            bottomplot=True,
            ):
    infile = fileutil.osimg(infile)
    infits = pyfits.open(infile)
    indir = os.path.split(infile)[0]
    head  = infits[1].header
    slist = infits[1].data

    if xmin == iraf.INDEF: xmin = None
    if xmax == iraf.INDEF: xmax = None
    if ymin == iraf.INDEF: ymin = None
    if ymax == iraf.INDEF: ymax = None
    if boxcar == iraf.INDEF: boxcar = None
    if bin == iraf.INDEF: bin = None
    if grid == iraf.INDEF: grid = None
    #if xlabel == 'INDEF': xlabel = r'$\AA$'
    #if ylabel == 'INDEF': ylabel = r'$erg/s/cm^2/\AA$'
    
    if plotoh:
        ohfile = iraf.osfn(ohfile)
    else:
        ohfile = None
    
    if plotres and rescurve == 'INDEF' and head.has_key('MDP_RESP'):
        rescurve = set_fname(head['MDP_RESP'], indir)
    else:
        rescurve = None
    
    if plotfit and sfdbfile == 'INDEF' and head.has_key('MDP_SFDB'):
        sfdbfile = set_fname(head['MDP_SFDB'], indir)
    if plotfit and sfdbfile != 'INDEF':
        fitfunc = sfdb.open(sfdbfile).get_func()
    else:
        fitfunc = None

    # set title
    if title == 'INDEF':
        if head.has_key('MDP_SFTP'):
            title = head['OBJECT'] + ' - ' + head['MDP_SFTP']
        else:
            title = head['OBJECT']

    twodspec.onedplot(slist, style, errorbar=errorbar,
                      xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax, yscale=yscale,
                      boxcar=boxcar, bin=bin,
                      ohfile=ohfile, rescurve=rescurve, error=ploterror, fitfunc=fitfunc,
                      grid=grid, legend=legend,
                      title=title, xlabel=xlabel, ylabel=ylabel, 
                      ohstyle=ohstyle, rstyle=rstyle, fstyle=fstyle, estyle=estyle,
                      redshift=redshift, restwl=restwl, linelist=linelist,
                      lstyle=lstyle, linealpha=linealpha, linewidth=linewidth,
                      bottomplot=bottomplot)
    
    if outfile != 'INDEF':
        pylab.savefig(outfile)
