"""
mdprotate.py - rotate MDP MEF file
2007.07.27. YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.2  2007/09/03 08:38:27  tomohiro
add mdpshift

Revision 1.1  2007/08/01 01:59:16  tomohiro
rotate MDP MEF file

"""
from pyraf import iraf
import pyfits
from iraftool import *

def mdprotate(input, output, rotation,
              xin=iraf.INDEF, yin=iraf.INDEF, xout=iraf.INDEF, yout=iraf.INDEF,
              ncols=iraf.INDEF, nlines=iraf.INDEF,
              interpolant='linear', boundary='nearest', constant=0.0,
              nxblock=512, nyblock=512, verbose=True):
    listi = fileutil.fileparse(input, check=True, ext=True)
    listr = fileutil.fileparse(output, ext=True, suffix=True)

    for filei, filer in zip(listi, listr):
        next = len(pyfits.open(fileutil.osimg(filei[0])))
        rflist = tmpfile.open('mdprotate_rflist')
        iflist = tmpfile.open('mdprotate_iflist')
        for i in range(next):
            rflist.append('%s_%02d'%(filer[0],i))
            iflist.append('%s[%d]'%(filei[0],i))
        iraf.rotate('@'+iflist.path, '@'+rflist.path, rotation,
                    xin=xin, yin=yin, xout=xout, yout=yout,
                    ncols=ncols, nlines=nlines,
                    interpolant=interpolant, boundary=boundary, constant=constant,
                    nxblock=nxblock, nyblock=nyblock, verbose=verbose)
        if os.path.exists(fileutil.osimg(filer[0])):
            os.unlink(fileutil.osimg(filer[0]))
        for r in rflist.list():
            data, header = pyfits.getdata(fileutil.osimg(r), header=True)
            pyfits.append(fileutil.osimg(filer[0]), data, header)
        rflist.remove()
        rflist.close()
        iflist.close()

def mdpshift(input, output, xshift, yshift,
             shifts_file='INDEF', interp_type='linear', boundary_type='constant', constant=0.0):
    listi = fileutil.fileparse(input, check=True, ext=True)
    listr = fileutil.fileparse(output, ext=True, suffix=True)
    
    for filei, filer in zip(listi, listr):
        next = len(pyfits.open(fileutil.osimg(filei[0])))
        rflist = tmpfile.open('mdpshift_rflist')
        iflist = tmpfile.open('mdpshift_iflist')
        for i in range(next):
            rflist.append('%s_%02d'%(filer[0],i))
            iflist.append('%s[%d]'%(filei[0],i))
        iraf.imshift('@'+iflist.path, '@'+rflist.path,
                     xshift=xshift, yshift=yshift,
                     shifts_file=shifts_file, interp_type=interp_type,
                     boundary_type=boundary_type, constant=constant)
        if os.path.exists(fileutil.osimg(filer[0])):
            os.unlink(fileutil.osimg(filer[0]))
        for r in rflist.list():
            data, header = pyfits.getdata(fileutil.osimg(r), header=True)
            pyfits.append(fileutil.osimg(filer[0]), data, header)
        rflist.remove()
        rflist.close()
        iflist.close()
    
