"""
mesoffset.py - pyraf wrapper for mesoffset
YOSHIKAWA, Tomohiro
$Id: mesoffset.py,v 1.1 2008-02-02 00:05:05 tomohiro Exp $
"""

import pylab
from iraftool import geomap
import sys

def mdpmesoffset1(istar_chip1, isky_chip1, rootname1, cfile, iretry1, iretry2, iinter1, iinter2, datadir,
                  glaphtitle=None, scale=20, xmin=0, ymin=0, xmax=2048, ymax=3584, dmin=-3, dmax=3
                  ):
    mesoffset1(istar_chip1, isky_chip1, rootname1, cfile, iretry1, iretry2, iinter1, iinter2)
    star_coo = rootaname1 + "_star.coo"
    mask_coo = rootaname1 + "_mask.coo"
    magvec1(star_coo, mask_coo,
            glaphtitle=None, scale=20, xmin=0, ymin=0, xmax=2048, ymax=3584, dmin=-3, dmax=3)
    
def magvec1(star_coo, mask_coo,
            glaphtitle=None, scale=20, xmin=0, ymin=0, xmax=2048, ymax=3584, dmin=-3, dmax=3):
    x0, y0 = loadcoo(star_coo)
    x1, y1 = loadcoo(mask_coo)
    magvec(x0, y0, x1, y1,
           glaphtitle=None, scale=20, xmin=0, ymin=0, xmax=2048, ymax=3584, dmin=-3, dmax=3)
    pylab.figure(1)
    print "difference graph name? (not save): ",
    fname1 = sys.stdin.readline()
    fname1 = fname1.strip()
    if len(fname1) > 0:
        pylab.savefig(fname1)
    pylab.figure(2)
    print "residual graph name? (not save): ",
    fname2 = sys.stdin.readline()
    fname2 = fname2.strip()
    if len(fname2) > 0:
        pylab.savefig(fname2)

def loadcoo(file):
    x = []
    y = []
    for line in open(file):
        if line[0] == '#': continue
        data = line.split()
        data_x = float(data[0])
        data_y = float(data[1])
        if data_x == 0 and data_y == 0: continue
        x.append(data_x)
        y.append(data_y)
    return pylab.array(x), pylab.array(y)

def magvec(x0s, y0s, x1s, y1s,
           glaphtitle=None, scale=20,
           xmin=0, ymin=0, xmax=2048, ymax=3584,
           dmin=-3, dmax=3 ):
    reflist = []
    inlist = []
    for x0, y0, x1, y1 in zip(x0s, y0s, x1s, y1s):
        reflist.append({'REF_X': x0, 'REF_Y': y0})
        inlist.append({'X': x1, 'Y': y1})
    geodb = geomap.geomap(reflist, inlist, fitgeometry='rotate')
    rotate = geodb.data['xrotation']
    
    rotate = rotate/180.0*pylab.pi
    x1s_r = x1s*pylab.cos(rotate) - y1s*pylab.sin(rotate)
    y1s_r = x1s*pylab.sin(rotate) + y1s*pylab.cos(rotate)

    reflist = []
    inlist = []
    for x0, y0, x1, y1 in zip(x0s, y0s, x1s_r, y1s_r):
        reflist.append({'REF_X': x0, 'REF_Y': y0})
        inlist.append({'X': x1, 'Y': y1})
    geodb = geomap.geomap(reflist, inlist, fitgeometry='shift')
    shift_x=-geodb.data['xshift']
    shift_y=-geodb.data['yshift']
    x1s_r   += shift_x
    y1s_r   += shift_y

    #xrs0 = x1s_r-x0s
    #yrs0 = y1s_r-y0s
    
    xps = [100]
    yps = [100]
    xrs = [1]
    yrs = [0]
    for xp, yp, xr, yr, in zip(x0s, y0s, x1s_r-x0s, y1s_r-y0s):
        xps.append(xp)
        yps.append(yp)
        xrs.append(xr)
        yrs.append(yr)

    pylab.figure(1)
    pylab.clf()
    pylab.quiver(xps, yps, xrs, yrs, scale=scale)
    pylab.text(100, 120, '1 pixel')
    pylab.axhline(int(ymax/2), xmin, xmax, color='b', linestyle='-.')
    if glaphtitle != None: pylab.title(glaphtitle)
    pylab.xlabel('X')
    pylab.ylabel('Y')
    pylab.xlim(xmin, xmax)
    pylab.ylim(ymin, ymax)

    pylab.figure(2)
    if glaphtitle != None: pylab.title(glaphtitle)
    pylab.clf()
    pylab.subplot(221)
    if glaphtitle != None: pylab.title(glaphtitle)
    pylab.plot(x0s, x1s_r-x0s, 'ko')
    pylab.xlabel('X')
    pylab.ylabel('X residual')
    pylab.xlim(xmin, xmax)
    pylab.ylim(dmin, dmax)
    pylab.subplot(222)
    pylab.plot(x0s, y1s_r-y0s, 'ko')
    pylab.xlabel('X')
    pylab.ylabel('Y residual')
    pylab.xlim(xmin, xmax)
    pylab.ylim(dmin, dmax)
    pylab.subplot(223)
    pylab.plot(y0s, x1s_r-x0s, 'ko')
    pylab.xlabel('Y')
    pylab.ylabel('X residual')
    pylab.xlim(ymin, ymax)
    pylab.ylim(dmin, dmax)
    pylab.subplot(224)
    pylab.plot(y0s, y1s_r-y0s, 'ko')
    pylab.xlabel('Y')
    pylab.ylabel('Y residual')
    pylab.xlim(ymin, ymax)
    pylab.ylim(dmin, dmax)
    #return xps, yps, xrs, yrs
