"""
_catio.py - astronomical catalog I/O functions
YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.2  2008/02/01 23:28:34  tomohiro
add "fixlen" type catalog
add function mopen()

"""

import os
import aladin,gaia,gator,mdp,spitzer,sextractor,hyperz,fixlen,datalist
import glob

def open(file, type=None, column=[], colen=[]):
    """open Catalog file and returns Catalog object
    file: Catalog file name
    type: catalog file type
    column,colen: column name and length (for fixlen)
    """
    if type == None:
        ext = os.path.splitext(file)[1]
    else:
        ext = None
    
    if ext == ".gaia" or type == "gaia":
        return gaia.open(file)
    elif type == "gator":
        return gator.open(file)
    elif ext == ".mdp" or type == "mdp":
        return mdp.open(file)
    elif ext == ".tbl" or type == "spitzer":
        return spitzer.open(file)
    elif type == "sex":
        return sextractor.open(file)
    elif ext == ".cat" or type == "hyperz":
        return hyperz.open(file)
    elif type == "fixlen":
        return fixlen.open(file, column, colen)
    else:
        raise TypeError, "file type %s is not supported"%(type)
        #return Catalog(file)

def write(data, file, wkeys=None):
    if type == None:
        ext = os.path.splitext(file)[1]
    else:
        ext = None
    
    if ext == ".gaia" or type == "gaia":
        return gaia.writeto(data, file, wkeys)
    elif type == "gator":
        return gator.writeto(data, file, wkeys)
    elif ext == ".mdp" or type == "mdp":
        return mdp.open(data, file, wkeys)
    elif ext == ".tbl" or type == "spitzer":
        return spitzer.writeto(data, file, wkeys)
    elif type == "sex":
        return sextractor.writeto(data, file, wkeys)
    elif ext == ".cat" or type == "hyperz":
        return hyperz.open(data, file, wkeys)
    else:
        raise TypeError, "file type %s is not supported"%(type)

def mopen(flist, type=None, nstr=['None','N/A'], nv=-1):
    """open multiple catalog and return merged record
    ## all catalogs must have the same columns ##
    flist: file list with wild card
    type: catalog type
    nstr: list of string for None value
    nv:   value for None value
    """
    catlist = glob.glob(flist)
    catlist.sort()
    outlist = []
    for fname in catlist:
        outlist += open(fname, type=type).readlines(nstr=nstr, nv=nv)
    outlist = datalist.DataList(outlist)
    return datalist.dl2record(outlist)

