# aladin.py
# Aladin tsv catalog libraries

import __builtin__
import gaia, _catio
import catalog
from astrotool import radec
from types import *

class Aladin(gaia.Gaia):
    """Aladin tsv catalog class
    """
    ## Operations
    def __init__(self, file=None, wcsdelim=' '):
        """function __init__
        initialize Aladin object
        file: Aladin catalog file name
        wcsdelim: delimeter for wcs data
        """
        self.name = file
        self.column = None
        if self.name == None: return
        self._file = __builtin__.open(self.name)
        
        self.header = []
        for line in self._file.readlines():
            ## set column name
            if(line[0] != '#' and line[0] != '\n' ):
                line = line.strip()
                self.column = line.split('\t')
                break

        self.wcsdelim = wcsdelim
        #print self.column
        self._file.close()

    def readline(self, nv=None, nstr=['None']):
        self._file = __builtin__.open(self.name)
        ### skip header
        while(True):
            line = self._file.readline()
            if line[0] == '-': break
        ### read data
        #print line
        while(True):
            line = self._file.readline()
            if len(line) <= 0: break
            if line[0] == "\r" or line[0] == "\n" or line[0] == '#': continue
            tmp_outline = []
            line = line.strip()
            ldata =  line.split('\t')
            if len(ldata) < len(self.column):
                ldata += [nstr[0]]*(len(self.column)-len(ldata))
            for col,data in zip(self.column, ldata):
                data = catalog.str2data(data, nv=nv, nstr=nstr)
                if type(data) == StringType:
                    if col == 'RAJ2000':
                        data = radec.hrstr2deg(data, sep=self.wcsdelim)
                    elif col == 'DEJ2000':
                        data = radec.degstr2deg(data, sep=self.wcsdelim)
                tmp_outline.append((col,data))
            yield dict(tmp_outline)

    def writeto(self, list, name=None, RAkey='RA', DECkey='Dec', IDkey='ID', wkeys=None):
        """write list to gaia file
        list: object list of dictionary
        name: output file name
        RAkey: name of key for RA in list
        DECkey: name of key for DEC inlist
        ** both RA and DEC should be in J2000 equatorial system **
        """
        delim = '\t' # delimiter for data
        if name == None: name == self.name
        self._file = __builtin__.open(name, 'w')
        if wkeys != None:
            column = [RAkey,DECkey,IDkey]+wkeys
        elif self.column == None:
            column = list[0].keys()
        else:
            column = self.column
        #print column
        RAindex = column.index(RAkey)
        del column[RAindex]
        DECindex = column.index(DECkey)
        del column[DECindex]
        IDindex = column.index(IDkey)
        del column[IDindex]
        #column.insert(0, IDkey)
        #column.insert(0, DECkey)
        #column.insert(0, RAkey)
        
        ## write header
        head = delim.join(['RAJ2000','DEJ2000','ID']+column)
        self._file.write(head+'\n')
        sep = []
        for hd in ['RAJ2000','DEJ2000','ID']+column:
            sep.append('-'*len(hd))
        self._file.write(delim.join(sep) + '\n')

        ## write data
        for line in list:
            data = []
            for key in [RAkey,DECkey,IDkey]+column:
                try:
                    data.append(str(line[key]))
                except KeyError:
                    data.append('N/A')
            self._file.write(delim.join(data) + '\n')
        self._file.close()

def open(file=None, wcsdelim=' '):
    """open aladin catalog file and returns gaia object
    file: aladin catalog file name
    """
    return Aladin(file, wcsdelim=wcsdelim)

if __name__ == "__main__":
    import gaia
    gcat = gaia.open('GOODSNspecz_zsel_z2025_04May2006')
    data = gcat.readlines()
    print data[0].keys()
    acat = open()
    acat.writeto(data, 'ika')

def convert(infile, outfile, RAkey='RA', DECkey='Dec', IDkey='ID', intype=None, wkeys=None):
    al = Aladin()
    incat = _catio.open(infile, type=intype)
    indata = incat.readlines()
    al.column = incat.column
    al.writeto(indata, outfile, RAkey, DECkey, IDkey, wkeys=wkeys)
