# datalist.py
# datalist manipulator functions
# 2006.06.28 YOSHIKAWA, Tomohiro
# $Log: not supported by cvs2svn $
# Revision 1.8  2008/07/28 03:02:21  tomohiro
# modify namesarch() to namesearch()
#
# Revision 1.7  2008/07/28 02:50:36  tomohiro
# modify namesarch_key() to namesearch_key()
#
# Revision 1.6  2008/06/20 04:02:36  tomohiro
# add parameter dup to namesearch_key to allow duplication in name
#
# Revision 1.5  2008/03/19 10:23:36  tomohiro
# modify to use numpy, add apppend_column() method to DataList, add function namesearch_key()
#
# Revision 1.4  2008/02/01 23:41:43  tomohiro
# add wkey parameter to printwiki()
# add nstr, nv parameter to str2data()
# add function namesearch()
#
# Revision 1.3  2007/09/03 08:42:22  tomohiro
# add printwiki()
#
# Revision 1.2  2007/07/17 02:29:17  tomohiro
# add rec2datalist(), dl2record()
#

#import numarray
import numpy
from numpy import rec
import __builtin__,sys

class DataList(__builtin__.list):
    def __init__(self, seq=None, header=None):
        self.header = header
        self._keys = []
        if seq != None:
            for key in seq[0].keys():
                self._keys.append(key)
            for data in seq:
                for key in self._keys:
                    if not data.has_key(key):
                        raise TypeError, "invalid format: key "+key+" doesn't exist in "+str(data)
            __builtin__.list.__init__(self, seq)
        else:
            __builtin__.list.__init__(self)
    def getarray(self, key):
        return getarray(self, key)
    def has_data(self, key, value):
        for data in self:
            if data[key] == value: return True
        return False
    def has_key(self, key):
        return self[0].has_key(key)
    def keys(self):
        return self._keys
    def getbykey(self, key, value):
        for data in self:
            if data[key] == value: return data
        return None
    def copy(self):
        """create deep copy"""
        datalist2 = []
        for data in self:
            data2 = data.copy()
            datalist2.append(data2)
        if self.header != None:
            return DataList(datalist2, self.header.copy())
        else:
            return DataList(datalist2)
    def xyshift(self, xshift, yshift, xname='X', yname='Y'):
        for data in self:
            data[xname] += xshift
            data[yname] += yshift

    def thresh(self, tname, tmax=None, tmin=None):
        tdata = numpy.array(self.getarray(tname))
        if tmax != None and tmin != None:
            index = numpy.where((tdata > tmin) * (tdata < tmax))
        elif tmax != None:
            index = numpy.where(tdata < tmax)
        elif tmin != None:
            index = numpy.where(tdata > tmin)
        else:
            raise ValueError, 'invalid tmax and tmin'
        return index

    def thresh_list(self, tname, tmax=None, tmin=None):
        tindex = self.thresh(tname, tmax, tmin)
        outlist = []
        for i in tindex[0]:
            outlist.append(self[i].copy())
        return DataList(outlist)

    def printwiki(self, wkeys=None):
        if wkeys == None:
            wkeys = self._keys
        # print header
        sys.stdout.write('|')
        for k in wkeys:
            sys.stdout.write(k+'|')
        sys.stdout.write('h\n')
        
        # print data
        for data in self:
            sys.stdout.write('|')
            for k in wkeys:
                try:
                    if data[k] < 0:
                        sys.stdout.write('-|')
                    elif data[k] < 0.1:
                        sys.stdout.write('%.3e|'%(data[k]))
                    else:
                        sys.stdout.write('%.3f|'%(data[k]))
                except TypeError:
                    sys.stdout.write('%s|'%(data[k]))
            sys.stdout.write('\n')

    def append_column(self, col, key):
        """append a new column
        col: array of new column (must be the same length as current datalist)
        key: key for new column
        """
        if len(self) != len(col):
            raise ValueError, 'new column must be the same length as current datalist'
        for line, data in zip(self, col):
            line[key] = data
        self._keys.append(key)

def getarray(datalist, key):
    retdata=[]
    for index in range(len(datalist)):
        retdata.append(datalist[index][key])
    return retdata

def rec2datalist(record, wkeys=None):
    """convert record to datalist
    """
    outlist = []
    for i in range(len(record)):
        outlist.append({})
    #if wkeys == None: wkeys = record._names
    if wkeys == None: wkeys = record.dtype.names
    for key in wkeys:
        values = record.field(key)
        for i in range(len(record)):
            outlist[i][key] = values[i]
    return DataList(outlist)

def dl2record(datalist):
    recarray = []
    for data in datalist:
        recarray.append(data.values())
    return rec.array(recarray, names=data.keys())

def str2data(data, nstr=['None'], nv=None):
    """
    data: data string
    nstr: list of strings to convert to nv
    nv:   value if the data is None
    """
    try:
        data = int(data)
    except ValueError:
        try:
            data = float(data)
        except ValueError:
            for ns in nstr:
                if data == ns:
                    data = nv
                    break
    return data

def namesearch(catlist, name, namekey='ID_K'):
    """search with name and make record
    catlist: input catalog (record)
    name: list of name
    namekey: the key name in catlist
    """
    outlist = []
    for n in name:
        datar = catlist[catlist[namekey] == n]
        #print len(datar), n
        if len(datar) != 1:
            print '%s not found'%(n)
            continue
        data = rec2datalist(datar)
        outlist += data
    return dl2record(DataList(outlist))

def namesearch_key(catlist, name, namekey='ID_K', dup=False):
    """search with name and make record
    catlist: input catalog (record)
    name: list of name
    namekey: the key name in catlist
    dup: allow duplication in name
    """
    outlist = []
    catdata = rec2datalist(catlist)
    if hasattr(name, 'tolist'):
        name = name.tolist()
    for data in catdata:
        n = data[namekey]
        try:
            i = name.index(n)
            outlist.append(True)
            #print name[i]
            if not dup:
                name = name[0:i]+name[i+1:]
        except ValueError:
            outlist.append(False)
    return numpy.array(outlist)

def namesearch_key2(catlist, name, namekey='ID_K', len=None, add_none=False):
    """search with name and make index list 
    catlist: input catalog (record)
    name: list of name to search
    namekey: the key name in catlist
    len: length to mach the key value
    add_none: add None to the index list if an object is not found
    """
    outlist = []
    catdata = rec2datalist(catlist)
    if hasattr(name, 'tolist'):
        name = name.tolist()
    iname = catlist[namekey].tolist()
    keylist = []
    for n in name:
        try:
            if len:
                k = iname.index(n[:len])
            else:
                k = iname.index(n)
            keylist.append(k)
        except ValueError:
            if add_none:
                keylist.append(None)
            else:
                print '%s not found'%(n)
    return numpy.array(keylist)

mp_enable=True
try:
    from multiprocessing import Pool
except ImportError:
    try:
        from processing import Pool
    except ImportError:
        mp_enable=False

if mp_enable:
    class ns2:
        def __init__(self, inlist):
            self.inlist = inlist
            
        def __call__(self, refname):
            try:
                return self.inlist.index(refname)
            except:
                return None

    def namesearch_key2_mp(inname, refname, mp=4):
        """search with name and make index list (multiprocessor)
        inname: list of name to search
        reference: list of name to search
        mp: # of processor (4)
        """
        p = Pool(mp)
        f = ns2(inname)
        return p.map(f, refname)

def catmerge_by_name(cat1, cat2, name1, name2, keys1, keys2):
    """merge two catalog by name
    cat1, cat2: catalogs to merge
    name1, name2: name to merge
    keys1, keys2: lists for merged catalog
    """
    outlist = []
    catdata1 = rec2datalist(cat1)
    catdata2 = rec2datalist(cat2)
    cat2names = cat2[name2].tolist()
    for data1 in catdata1:
        outdata = []
        for k1 in keys1:
            outdata.append((k1, data1[k1]))
        try:
            i = cat2names.index(data1[name1])
            for k2 in keys2:
                outdata.append((k2, catdata2[i][k2]))
        except ValueError:
            for k2 in keys2:
                outdata.append((k2, -1))
        outlist.append(dict(outdata))
    
    return dl2record(outlist)
