# gaia.py
# gaia catalog utility libraries
# $Log: not supported by cvs2svn $
# Revision 1.2  2007/07/17 02:31:02  tomohiro
# add writerecord()
#

import __builtin__
import catalog
import datalist

class Gaia(catalog.Catalog):
    """Gaia catalog class
    """

    ## Operations
    def __init__(self, file=None):
        """function __init__
        initialize Gaia object
        file: Gaia catalog file name
        """
        self.name = file
        self.column = None
        if self.name == None: return
        self._file = __builtin__.open(self.name)

        self.header = []
        for line in self._file.readlines():
            ## set header
            if line[:1] == '#%':
                line = line[2:].strip()
                data = line.split('=')
                key = data[0].strip()
                vals = data[1].split('#')
                value = str2data(vals[0].strip())
                self.header.append((key,value))
            ## set column name
            elif(line[0] != '#' and line[0] != '\n' ):
                line = line.strip()
                self.column = line.split()
                break
        #print self.column
        self._file.close()

    def readlines0(self):
        self._file = __builtin__.open(self.name)
        output_list = []
        ### skip header
        while(True):
            line = self._file.readline()
            if line[0] == '-': break
        ### read data
        #print line
        while(True):
            line = self._file.readline()
            if len(line) <= 0: break
            if line[0] == "\r" or line[0] == "\n" or line[0] == '#': continue
            tmp_outline = []
            line = line.strip()
            for col,data in zip(self.column, line.split()):
                data = catalog.str2data(data)
                tmp_outline.append((col,data))
            output_list.append(dict(tmp_outline))
        return datalist.DataList(output_list)


    def readline(self, nv=None, nstr=['None']):
        self._file = __builtin__.open(self.name)
        ### skip header
        while(True):
            line = self._file.readline()
            if line[0] == '-': break
        ### read data
        #print line
        while(True):
            line = self._file.readline()
            if len(line) <= 0: break
            if line[0] == "\r" or line[0] == "\n" or line[0] == '#': continue
            tmp_outline = []
            line = line.strip()
            ldata =  line.split()
            if len(ldata) < len(self.column):
                ldata += [nstr[0]]*(len(self.column)-len(ldata))
            for col,data in zip(self.column, ldata):
                data = catalog.str2data(data, nv=nv, nstr=nstr)
                tmp_outline.append((col,data))
            yield dict(tmp_outline)
    
    def writeto(self, list, name=None, wkeys=None):
        """write list to gaia file
        list: object list of list or dictionary
        name: output file name
        wkeys: key name list to write
        """
        delim = '\t' # delimiter for data
        if name == None: name == self.name
        self._file = __builtin__.open(name, 'w')
        if wkeys != None:
            column = wkeys
            listisdic = True
        else:
            try:
                tmp_col = list[0].keys()  # list is list of dictionary
                if(self.column == None):
                    column = tmp_col
                else:
                    column = self.column
                listisdic = True
            except AttributeError:  # list is list of list
                if self.column == None:
                    rn = range(len(list[0]))
                    column = []
                    for i in rn:
                        column.append('%d' % (i))
                else:
                    column = self.column
                listisdic = False
        ## write header
        head = delim.join(column)
        self._file.write(head+'\n')
        self._file.write('-'*len(head) + '\n')

        ## write data
        for line in list:
            data = []
            if(listisdic):
                for key in column:
                    try:
                        data.append(str(line[key]))
                    except KeyError:
                        data.append('N/A')
            else:
                for elem in line:
                    data.append(str(elem))
            self._file.write(delim.join(data) + '\n')
        self._file.close()
        
def open(file=None):
    """open gaia catalog file and returns gaia object
    file: gaia catalog file name
    """
    return Gaia(file)

def writedata(list, file, wkeys=None):
    """write data in the list of dictionaries to gaia file
    list: a list of lists of dictionaries
    file: output file name
    """
    gfile = Gaia()
    gfile.writeto(list, file, wkeys)

def writerecord(record, file, wkeys=None):
    """write data in the list of dictionaries to gaia file
    list: a list of lists of dictionaries
    file: output file name
    """
    if wkeys == None:
        wkeys = record.dtype.names
    outlist = datalist.rec2datalist(record, wkeys)
    writedata(outlist, file, wkeys)

if __name__ == "__main__":
    cat = open('GOODSNspecz_zsel_z2025_04May2006')
    data = cat.readlines()
    print data[0].keys()
    cat.writeto(data, 'tako')

