"""radec.py - functions for RA/DEC
YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.4  2008/09/07 13:33:11  tomohiro
remove circular import of datalist

Revision 1.3  2008/09/04 03:48:49  tomohiro
add sep parameter to degstr2deg() and hrstr2deg()

"""
from RO.Astro.Sph import *
from RO.Astro.Tm import *
from RO import MathUtil
import math, numpy

def degstr2deg(degstr, sep=':'):
    """convert degree string to degree
    degstr: dd:mm:ss
    sep: separator
    """
    data = degstr.split(sep)
    d = int(data[0])
    m = int(data[1])
    s = float(data[2])
    if d < -90 or d > 90: raise ValueError, 'degree is out of range'
    if m < 0   or m > 60: raise ValueError, 'minute is out of range'
    if s < 0   or s > 60: raise ValueError, 'second is out of range'
    
    if d < 0:
        return d - m/60. - s/3600.
    else:
        return d + m/60. + s/3600.

def hrstr2deg(hrstr, sep=':'):
    """convert hour string to degree
    degstr: hh:mm:ss
    sep: separator
    """
    data = hrstr.split(sep)
    h = int(data[0])
    m = int(data[1])
    s = float(data[2])
    if h < 0 or h > 24: raise ValueError, 'hour is out of range'
    if m < 0 or m > 60: raise ValueError, 'minute is out of range'
    if s < 0 or s > 60: raise ValueError, 'second is out of range'
    
    return (h + m/60. + s/3600.)/24.*360

def ckdegstr(degstr):
    """check degree string
    degstr: dd:mm:ss
    """
    data = degstr.split(':')
    d = int(data[0])
    m = int(data[1])
    s = float(data[2])
    if d < -90 or d > 90: raise ValueError, 'degree is out of range'
    if m < 0   or m > 60: raise ValueError, 'minute is out of range'
    if s < 0   or s > 60: raise ValueError, 'second is out of range'
    return True

def chhrstr(hrstr):
    """convert hour string to degree
    degstr: hh:mm:ss
    """
    data = hrstr.split(':')
    h = int(data[0])
    m = int(data[1])
    s = float(data[2])
    if h < 0 or h > 24: raise ValueError, 'hour is out of range'
    if m < 0 or m > 60: raise ValueError, 'minute is out of range'
    if s < 0 or s > 60: raise ValueError, 'second is out of range'
    return True

def degstr2degl(inlist, degkey='DEC', degkey_out=None):
    """
    convert nn:nn:nn.n degree string to float degree
    """
    for data in inlist:
        if degkey_out != None:
            data[degkey_out] = degstr2deg(data[degkey])
        else:
            data[degkey] = degstr2deg(data[degkey])

def hrstr2degl(inlist, hrkey='DEC', hrkey_out=None):
    """
    convert nn:nn:nn.n hour string to float degree
    """
    for data in inlist:
        if hrkey_out != None:
            data[hrkey_out] = hrstr2deg(data[hrkey])
        else:
            data[hrkey] = hrstr2deg(data[hrkey])

def deg2degstr(deg, sep=':', precision=None):
    try:
        len(deg)
        outlist = []
        for d in deg:
            outlist.append(deg2degstr(d, sep, precision))
        return outlist
    except TypeError:
        pass
    d = int(math.floor(deg))
    m = int(math.floor((deg - d)*60))
    s = ((deg - d)*60 - m)*60
    if precision != None:
        pre = '%%.%df'%(precision)
    else:
        pre = '%f'
    if s < 10:
        return ('%02d%s%02d%s0'+pre)%(d,sep,m,sep,s)
    else:
        return ('%02d%s%02d%s'+pre)%(d,sep,m,sep,s)

def deg2hrstr(deg, sep=':', precision=None):
    try:
        len(deg)
        outlist = []
        for d in deg:
            outlist.append(deg2hrstr(d, sep, precision))
        return outlist
    except TypeError:
        pass
    hr = deg/360.*24.
    h = int(math.floor(hr))
    m = int(math.floor((hr - h)*60))
    s = ((hr - h)*60 - m)*60
    if precision != None:
        pre = '%%.%df'%(precision)
    else:
        pre = '%f'
    if s < 10:
        return ('%02d%s%02d%s0'+pre)%(h,sep,m,sep,s)
    else:
        return ('%02d%s%02d%s'+pre)%(h,sep,m,sep,s)

def wcsdist(rabase, decbase, ra, dec):
    """
    calculate distance between (rabase, decbase) and (ra, dec)
    arguments are given in units of degree
    numpy array is accepted for arguments
    """
    dra = (ra-rabase)*numpy.cos(decbase/180.*numpy.pi)
    ddec = dec-decbase
    return numpy.sqrt(dra*dra+ddec*ddec)
