"""
 reddening.py - calculate reddening
 2007.06.07 YOSHIKAWA, Tomohiro
 $Id: reddening.py,v 1.3 2009-01-09 01:07:26 tomohiro Exp $
"""
import numpy
def calzetti_func(lmd, Rv=4.05):
    try:
        l = len(lmd)
    except TypeError:
        if lmd < 0.63:
            return __cl1(lmd, Rv)
        else:
            return __cl2(lmd, Rv)
    outlist = []
    for lm in lmd:
        outlist.append(calzetti_func(lm, Rv))
    return numpy.array(outlist)

def __cl1(lmd, Rv):
    return 2.659*(-2.156 + 1.509/lmd - 0.198/lmd**2 + 0.011/lmd**3) + Rv
def __cl2(lmd, Rv):
    return 2.659*(-1.857 + 1.040/lmd) + Rv

class Reddening:
    def __init__(self, Rv=4.05, Av=1.0, kl=calzetti_func):
        """
        Av = Rv * E(B-V)
        where: Rv = 3.1 (general)
                  = 2.72 (SMC)
                  = 4.05 (Calzetti)
        A_lambda = kl(lambda) * Av / Rv
        """
        self.Rv = Rv
        self.Av = Av
        self.kl = kl

    def Al(self, lmd, Av=None):
        if Av == None: Av = self.Av
        return self.kl(lmd) * Av / self.Rv
        
    def mag(self, magi, lmd, Av=None):
        """
        magi: intrinsic magnitude
        lmd: wavelength (um) of magi
        Av:   use dafault if None
        """
        return magi + self.Al(lmd, Av)

    def flux(self, fluxi, lmd, Av=None):
        """
        fluxi: intrinsic flux
        lmd: wavelength (um) of fluxi
        Av:   use dafault if None
        """
        return fluxi / 10**(0.4*self.Al(lmd, Av))
    
class Calzetti(Reddening):
    def __init__(self, Av=1.0):
        Reddening.__init__(self, 4.05, 1.0, calzetti_func)
    
