# geomap.py - database file for geomap & geotran
# $Id: geomap.py,v 1.2 2007-09-03 08:44:14 tomohiro Exp $
from pyraf import iraf
import time,os
import pyfits
from astrocat import datalist

class GeoMapDB:
    def __init__(self, database=None, transform=None):
        if database != None and transform != None:
            self.load(database, transform)

    def load(self, database, transform):
        self.transform = None
        self.data = []
        self.surface1 = None
        self.surface2 = None
        db_f = open(database)
        
        while(True):
            line = db_f.readline()
            if len(line) == 0: break
            # skip
            #print line
            if line[0] == '\n' or line[0] == '#': continue
            key, value = line.split()[:2]
            # transform search
            if key == 'begin':
                #print value
                if self.transform != None: break
                if value == transform:
                    self.transform = transform
                continue
            elif self.transform == None: continue
            # surface1
            if self.surface1 == None and key == 'surface1':
                self.surface1 = []
                for num in range(int(value)):
                    data = db_f.readline().split()
                    self.surface1.append((float(data[0]), float(data[1])))
                continue
            # surface2
            if self.surface2 == None and key == 'surface2':
                self.surface2 = []
                for num in range(int(value)):
                    data = db_f.readline().split()
                    self.surface2.append((float(data[0]), float(data[1])))
                continue

            # data
            self.data.append((key, datalist.str2data(value)))
            #print key, value

        if self.transform == None:
            raise ValueError, 'transform name not found: '+transform
        
        self.data = dict(self.data)
    
    def writeto(self, database, mode='a'):
        db_f = open(database, mode)
        db_f.write(time.strftime('# %c - generated by geomap.py\n'))

        # begin
        db_f.write("begin\t%s\n"%self.transform)
        # data
        for key, value in self.data.iteritems():
            db_f.write("\t%s\t%s\n"%(key,str(value)))
        
        # surface1
        if self.surface1 != None:
            db_f.write("\tsurface1\t%d\n"%(len(self.surface1)))
            for val1, val2 in self.surface1:
                db_f.write("\t\t\t%e\t%e\n"%(val1,val2))
        
        # surface2
        if self.surface2 != None:
            db_f.write("\tsurface2\t%d\n"%(len(self.surface2)))
            for val1, val2 in self.surface2:
                db_f.write("\t\t\t%e\t%e\n"%(val1,val2))
        
        db_f.close()

def loadlist(dbfile):
    dbnames = []
    for line in open(dbfile):
        if (line[0] == '#' or line[0] == '\n'): continue
        key, value = line.split()[:2]
        if key == 'begin':
            dbnames.append(value)
    dblist = []
    for db in dbnames:
        dblist.append(GeoMapDB(dbfile, db))
    return dblist

def geomap(reflist, inlist, trname=None,
           ref_x='REF_X', ref_y='REF_Y', in_x='X', in_y='Y',
           xsize=10000, ysize=10000, **kwargs):
    tmp_coo = iraf.osfn(iraf.mktemp("tmp$wcsmsc_coo"))
    tmp_db = iraf.osfn(iraf.mktemp("tmp$wcsmsc_db"))

    if trname == None:
        trname = tmp_coo
    
    coo_f = open(tmp_coo, 'w')
    for refdata, indata in zip(reflist, inlist):
        coo_f.write("%s %s %f %f\n"%(refdata[ref_x], refdata[ref_y], indata[in_x], indata[in_y]))
    coo_f.close()
        
    iraf.geomap(tmp_coo, tmp_db, 1, xsize, 1, ysize, transforms=trname, **kwargs)
    database = GeoMapDB(tmp_db, trname)
    
    if os.access(tmp_db, os.F_OK): os.unlink(tmp_db)
    if os.access(tmp_coo, os.F_OK): os.unlink(tmp_coo)

    return database

def geotran(inimg, outimg, database, gtrmask=None, **kwargs):
    if database != None:
        #print database.data
        tmp_db = iraf.osfn(iraf.mktemp("tmp$wcsmsc_db"))
        database.writeto(tmp_db, 'w')
        trname = database.transform
    else:
        tmp_db = ''
        trname = ''
    
    try:
        iraf.geotran(inimg, outimg, tmp_db, trname, **kwargs)
#                     boundary='constant', constant=0.0,
    except:
        combine = False
        if os.access(tmp_db, os.F_OK): iraf.rename(tmp_db, inimg+".db")
        print "failed to geotran:",inimg
        raise
    
    ### geotran for mask file
    fname, imindex = imsplit(inimg)
    infits = pyfits.open(fname+".fits")
    hdr = infits[imindex].header
    if(hdr.has_key('BPM') and gtrmask != None):
        if hdr['BPM'] != gtrmask and os.access(gtrmask+".fits", os.F_OK):
            print 'delete',gtrmask
            os.unlink(gtrmask+".fits")
        try:
            if hdr['BPM'] != gtrmask:
                iraf.imcopy(hdr['BPM'], gtrmask)
            iraf.chpixtype(gtrmask, gtrmask, "real", ver=False)
            iraf.geotran(gtrmask, gtrmask, tmp_db, trname, **kwargs)
#                         boundary='constant', constant=1, 
            iraf.imreplace(gtrmask, 1, lower=1e-8)
            iraf.chpixtype(gtrmask, gtrmask, "int", ver=False)
        except:
            combine = False
            print "failed to geotran:", gtrmask
            if os.access(tmp_db, os.F_OK): iraf.rename(tmp_db, inimg+".db")
            raise
        hdul = pyfits.open(outimg+'.fits', mode='update')
        hdul[0].header['BPM'] = gtrmask
        if(hdul[0].header.has_key('NCOMBINE')): del hdul[0].header['NCOMBINE']
        hdul.close()
    
    if os.access(tmp_db, os.F_OK): os.unlink(tmp_db)

def geoxytran(inlist, database,
              inx='X', iny='Y', tranx='TR_X', trany='TR_Y'):
    tmp_inxy = iraf.osfn(iraf.mktemp("tmp$wcsmsc_inxy"))
    tmp_tranxy = iraf.osfn(iraf.mktemp("tmp$wcsmsc_tranxy"))
    tmp_db = iraf.osfn(iraf.mktemp("tmp$wcsmsc_db"))
    database.writeto(tmp_db, 'w')

    inxy_f = open(tmp_inxy, 'w')
    for data in inlist:
        inxy_f.write('%f %f\n'%(data[inx], data[iny]))
    inxy_f.close()
    iraf.geoxytran(tmp_inxy, tmp_tranxy, tmp_db, database.transform, direction='backward')
    xlist = []
    ylist = []
    for line in open(tmp_tranxy):
        if (line[0] == '#' or line[0] == '\n'): continue
        x, y = line.split()[:2]
        xlist.append(float(x))
        ylist.append(float(y))
    outlist = []
    
    for x, y, data in zip(xlist, ylist, inlist):
        outdata = []
        for k,v in data.iteritems():
            outdata.append((k,v))
        outdata.append((tranx, x))
        outdata.append((trany, y))
        outlist.append(dict(outdata))
    
    if os.access(tmp_inxy, os.F_OK): os.unlink(tmp_inxy)
    if os.access(tmp_tranxy, os.F_OK): os.unlink(tmp_tranxy)
    return outlist

def imsplit(imgname):
    data = imgname.split("[")
    filename = data[0]
    if len(data) > 1:
        imnum = int(data[1].strip("]"))
    else:
        imnum = 0
    return filename, imnum
