# object.py
# classes for object
# 2006.06.03. YOSHIKAWA, Tomohiro
from mcsmdp.shape import *
from mcsmdp.error import *
import astrocat

class Object(Shape):
    """class Object
    """
    data = None
    def __init__(self, x, y, coord='image', coord_unit='default',
                 mark='circle', size=None, size_unit='default', color='yellow',
                 id=None, groupid=None, data=None, **kwargs):
        """constructor
        x,y: X, Y coordinate
        coord: coordinate name for X, Y (image, fk5, ...)
        coord_unit: (default, degree, radian, physical, image)
        mark: type of mark (circle, box, diamond, cross, x, arrow, boxcircle)
        size: size of mark
        color: color of mark
        id: shape identifier
        data: other data about object
        """
        if((mark == 'circle' or  mark == 'box') and size != None):
            Shape.__init__(self, x, y, coord=coord, coord_unit=coord_unit,
                           type=mark, size_x=size, size_y=size, size_unit=size_unit, color=color,
                           id=id, groupid=groupid, **kwargs)
        else:
            Shape.__init__(self, x, y, coord=coord, coord_unit=coord_unit,
                           type='point', point=mark, color=color,
                           id=id, groupid=groupid, **kwargs)
        self.data = data
    
    def setlabel(self, key):
        if self.data.has_key(key):
            labeltext=self.data[key]
        else:
            labeltext=''
        if self.regprops.has_key('text'):
            self.regprops['text']="{%s}"%labeltext
        else:
            tmp_props=[]
            for k,v in self.regprops.iteritems():
                tmp_props.append((k,v))
            tmp_props.append(('text', "{%s}"%labeltext))
            self.regprops = dict(tmp_props)

    def getid(self):
        return self.id

class ObjectList(ShapeList):
    """class ObjectList
    """
    def __init__(self):
        ShapeList.__init__(self)
        self.coord='image'
        self.cu='i'
        self.mark='circle'
        self.size=None
        self.su='i'
        self.color='yellow'
        self.groupid=''
        self.list=[]
    def loaddata(self, datalist, coord='image', coord_unit='default', xname=None, yname=None, group=''):
        self.coord=coord
        self.cu=getcu(coord_unit)
        self.groupid=group
        index=0
        for (x, y, xname, yname), data in zip(self._loadcoord(datalist, coord=coord, xname=xname, yname=yname), datalist):
            #del data[xname]
            #del data[yname]
            idstring = "%s%04i"%(group, index)
            index=index+1
            obj = Object(x, y, mark=self.mark, size=self.size, color=self.color,
                         coord=coord, coord_unit=coord_unit, id=idstring, groupid=group, data=data)
            self.list.append(obj)
    
    def setprop(self, mark=None, size=None, color=None, **kwargs):
        if mark != None:
            self.mark=mark
        if size != None:
            self.size=size
        if color != None:
            self.color=color
        if((mark == 'circle' or  mark == 'box') and size != None):
            ShapeList.setprop(self, type=mark, size_x=size, size_y=size, color=color, **kwargs)
        else:
            ShapeList.setprop(self, type='point', point=mark, color=color, **kwargs)

    def setlabel(self, key):
        for i in range(len(self.list)):
            self.list[i].setlabel(key)
    
    def setid(self, key):
        for i in range(len(self.list)):
            self.list[i].setid(key)

    def getbyid(self, id):
        """method getbyid
        get instance of object by id label
        id: id to get object
        """
        for index in range(len(self.list)):
            if self.list[index].getid() == id:
                return self.list[index]
    
    def search(self, key, value):
        """method search
        search the value from the data having the key
        """
        retlist = []
        for index in range(len(self.list)):
            if self.list[index].data.has_key(key):
                if self.list[index].data[key] == value:
                    retlist.append(self.list[index])
        return retlist
    
def LoadList(file, type='gaia', group=None, **kwargs):
    """function LoadList
    constructor for ObjectList
    file: file name to load
    type: file type ('gaia', 'gator', 'spitzer', 'sex')
    """
    catfile = astrocat.open(file, type=type)

    if(group == None):
        import os.path
        group = os.path.basename(file).split('.')[0]
    
    datalist = catfile.readlines()
    list = ObjectList()
    list.loaddata(datalist, group=group, **kwargs)
    return list
